Package: dynamicpv
Title: Evaluates Present Values and Health Economic Models with Dynamic
        Pricing and Uptake
Version: 0.4.2
Authors@R: c(
    person("Dominic", "Muston", , "dom.muston@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-4876-7940")),
    person("John", "Blischak", role = c("ctb"),
           comment = c(ORCID = "0000-0003-2634-9879")),
    person("Merck & Co., Inc., Rahway, NJ, USA and its affiliates", role = c("cph", "fnd"))
    )
Description: The goal of 'dynamicpv' is to provide a simple way to calculate (net) present values and outputs from health economic models (especially cost-effectiveness and budget impact) in discrete time that reflect dynamic pricing and dynamic uptake. Dynamic pricing is also known as life cycle pricing; dynamic uptake is also known as multiple or stacked cohorts, or dynamic disease prevalence. Shafrin (2024) <doi:10.1515/fhep-2024-0014> provides an explanation of dynamic value elements, in the context of Generalized Cost Effectiveness Analysis, and Puls (2024) <doi:10.1016/j.jval.2024.03.006> reviews challenges of incorporating such dynamic value elements. This package aims to reduce those challenges.
License: GPL (>= 3)
URL: https://MSDLLCpapers.github.io/dynamicpv/,
        https://github.com/MSDLLCpapers/dynamicpv
BugReports: https://github.com/MSDLLCpapers/dynamicpv/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: dplyr, heemod, readr, rlang, tidyr
Suggests: flexsurv, ggplot2, knitr, lubridate, purrr, rmarkdown,
        scales, testthat (>= 3.0.0), usethis
Config/testthat/edition: 3
LazyData: true
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-29 21:53:42 UTC; dom
Author: Dominic Muston [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4876-7940>),
  John Blischak [ctb] (ORCID: <https://orcid.org/0000-0003-2634-9879>),
  Merck & Co., Inc., Rahway, NJ, USA and its affiliates [cph, fnd]
Maintainer: Dominic Muston <dom.muston@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-30 08:30:31 UTC
Built: R 4.5.2; ; 2026-05-01 18:39:34 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("+",dynpv)
S3method("-",dynpv)
S3method(mean,dynpv)
S3method(print,dynpv_summary)
S3method(summary,dynpv)
export(addprod)
export(dynpv)
export(futurepv)
export(get_dynfields)
export(get_param_value)
export(ncoh)
export(ntimes)
export(sum_by_coh)
export(total)
export(trim_vec)
export(uptake)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,as_tibble)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,join_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarize)
importFrom(tidyr,expand_grid)
importFrom(tidyr,pivot_wider)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# dynamicpv: Evaluate present values and cost-effectiveness with dynamic pricing and uptake <img src="man/figures/logo.png" align="right" height="139" />

<!-- badges: start -->

[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![R-CMD-check](https://github.com/MSDLLCpapers/dynamicpv/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/MSDLLCpapers/dynamicpv/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/MSDLLCpapers/dynamicpv/graph/badge.svg)](https://app.codecov.io/gh/MSDLLCpapers/dynamicpv)
<!-- badges: end -->

The goal of ‘dynamicpv’ is to provide a simple way to calculate (net)
present values and outputs from health economic models (especially
cost-effectiveness and budget impact) in discrete time that reflect
dynamic pricing and dynamic uptake.

Dynamic pricing is also known as life cycle pricing; dynamic uptake is
also known as multiple or stacked cohorts, or dynamic disease
prevalence. [Shafrin (2024)](https://doi.org/10.1515/fhep-2024-0014)
provides an explanation of dynamic value elements, in the context of
Generalized Cost Effectiveness Analysis, and [Puls
(2024)](https://doi.org/10.1016/j.jval.2024.03.006) reviews challenges
of incorporating such dynamic value elements. This package aims to
reduce those challenges.

Through the
[dynpv()](https://msdllcpapers.github.io/dynamicpv/reference/dynpv.html)
function, the package provides calculations of the present values of
costs, life years, QALYs or other payoffs allowing for dynamic uptake
and dynamic pricing. The starting point is a conventional cohort
cost-effectiveness model, such as one computed using the
[heemod](https://cran.r-project.org/package=heemod) package.

## Installation

The package should shortly be available on
[CRAN](https://cran.r-project.org/web/packages/index.html). Installing
from here is simplest, once available.

``` r
# CRAN installation - when available
install.packages("dynamicpv")
```

You can install the development version of *dynamicpv* from
[GitHub](https://github.com/) as follows.

``` r
# Install devtools package if not already installed
install.packages("devtools")

# Install dynamicpv using pak from MSD repository
pak::pak("MSDLLCpapers/dynamicpv")
```

Note that the above does not install vignettes. Vignettes may be viewed
on the [package website](https://msdllcpapers.github.io/dynamicpv/) or
by instead running the following.

``` r
devtools::install_github("MSDLLCpapers/dynamicpv", build_vignettes=TRUE)
```

You should then load the package, alongside some other packages used
here.

``` r
# Load the dynamicpv package
library(dynamicpv)

# Recommended others
library(dplyr)
library(heemod)
library(ggplot2)
library(tidyr)
```

## How to use

There are four vignettes provided.

1.  The [Mathematical
    Framework](https://msdllcpapers.github.io/dynamicpv/articles/math-framework.html)
    vignette (`vignette("math-framework")`) describes the motivation and
    mathematical framework behind this package.

2.  The [Net Present
    Value](https://msdllcpapers.github.io/dynamicpv/articles/net-present-value.html)
    vignette (`vignette("net-present-value")`) describes how the package
    can be used to derive Net Present Values of simple cashflows with
    dynamic pricing and/or dynamic uptake.

3.  The [Cost-Effectiveness
    Applications](https://msdllcpapers.github.io/dynamicpv/articles/cost-effectiveness-applications.html)
    (`vignette("cost-effectiveness-applications")`) vignette describes
    how, given a static cost-effectiveness model (single cohort, prices
    constant in real terms), Incremental Cost-Effectiveness Ratios may
    be calculated that allow for dynamic pricing and/or dynamic uptake.
    This is illustrated by taking an example three-state partitioned
    survival model of a new intervention for an oncology indication
    compared to standard of care, with stated assumptions for how
    pricing and uptake may be dynamic. Nominal and Real ICERs are
    plotted over time, illustrating the impact of the loss of
    exclusivity points for each intervention.

4.  The [Budget Impact
    Applications](https://msdllcpapers.github.io/dynamicpv/articles/budget-impact-applications.html)
    (`vignette("budget-impact-applications")`) vignette describes how,
    given a static cost-effectiveness model (single cohort, prices
    constant in real terms) and dynamic uptake, a budget impact can be
    readily calculated. The calculations are repeated with the dynamic
    pricing assumptions to illustrate how easily this can be done, and
    examine the differences between a budget impact assessment with and
    without dynamic pricing.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

# Numeric notation in this document
# With thanks to https://stackoverflow.com/questions/18965637/set-global-thousand-separator-on-knitr/18967590#18967590
knitr::knit_hooks$set(
    inline = function(x) {
        if(!is.numeric(x)){
            x
        } else if(x<100) {
            prettyNum(round(x, 3), big.mark=",")
        } else {
            prettyNum(round(x, 0), big.mark=",")
        }
    }
)

## ----setup, echo=TRUE, message=FALSE------------------------------------------
library(dplyr)
library(ggplot2)
library(lubridate)
library(flexsurv)
library(heemod)
library(tidyr)
library(dynamicpv)

## ----constants----------------------------------------------------------------
# Time constants
days_in_year <- 365.25
days_in_week <- 7
cycle_years <- days_in_week / days_in_year # Duration of a one week cycle in years

# Time horizon (years) and number of cycles
thoz <- 20
Ncycles <- ceiling(thoz/cycle_years)

# Real discounting
disc_year <- 0.03 # Per year
disc_cycle <- (1+disc_year)^(cycle_years) - 1 # Per cycle

# Price inflation
infl_year <- 0.025 # Per year
infl_cycle <- (1+infl_year)^(cycle_years) - 1 # Per cycle

# Nominal discounting
nomdisc_year <- (1+disc_year)*(1+infl_year) - 1
nomdisc_cycle <- (1+nomdisc_year)^(cycle_years) - 1 # Per cycle

## ----heemod-------------------------------------------------------------------
# State names
state_names = c(
  progression_free = "PF",
  progression = "PD",
  death = "Death"
  )

# PFS distribution for SoC with Exp() distribution and mean of 50 weeks
surv_pfs_soc <- heemod::define_surv_dist(
  distribution = "exp",
  rate = 1/50
)

# OS distribution for SoC with Lognorm() distribution, meanlog = 4.5, sdlog = 1
# This implies a mean of exp(4 + 0.5 * 1^2) = exp(4.5) = 90 weeks
surv_os_soc <- heemod::define_surv_dist(
  distribution = "lnorm",
  meanlog = 4,
  sdlog = 1
)

# PFS and OS distributions for new
surv_pfs_new <- heemod::apply_hr(surv_pfs_soc, hr=0.5)
surv_os_new <- heemod::apply_hr(surv_os_soc, hr=0.6)

# Define partitioned survival model, soc
psm_soc <- heemod::define_part_surv(
  pfs = surv_pfs_soc,
  os = surv_os_soc,
  terminal_state = FALSE,
  state_names = state_names
  )

# Define partitioned survival model, soc
psm_new <- heemod::define_part_surv(
  pfs = surv_pfs_new,
  os = surv_os_new,
  terminal_state = FALSE,
  state_names = state_names
  )

# Parameters
params <- heemod::define_parameters(
  # Discount rate
  disc = disc_cycle,
  # Disease management costs
  cman_pf = 80,
  cman_pd = 20,
  # Drug acquisition costs - the SoC regime only uses SoC drug, the New regime only uses New drug
  cdaq_soc = dispatch_strategy(
    soc = 400,
    new = 0
  ),
  cdaq_new = dispatch_strategy(
    soc = 0,
    new = 1500
  ),
  # Drug administration costs
  cadmin = dispatch_strategy(
    soc = 50,
    new = 75
  ),
  # Adverse event risks
  risk_ae = dispatch_strategy(
    soc = 0.08,
    new = 0.1
  ),
  # Adverse event average costs
  uc_ae = 2000,
  # Subsequent treatments
  csubs = dispatch_strategy(
    soc = 1200,
    new = 300
  ),
  # Health state utilities
  u_pf = 0.8,
  u_pd = 0.6,
)

# Define PF states
state_PF <- heemod::define_state(
  # Costs for the state
  cost_daq_soc = discount(cdaq_soc, disc_cycle),
  cost_daq_new = discount(cdaq_new, disc_cycle),
  cost_dadmin = discount(cadmin, disc_cycle),
  cost_dman = discount(cman_pf, disc_cycle),  
  cost_ae = risk_ae * uc_ae,
  cost_subs = 0,
  cost_total = cost_daq_soc + cost_daq_new + cost_dadmin + cost_dman + cost_ae + cost_subs,
  # Health utility, QALYs and life years
  pf_year = discount(cycle_years, disc_cycle),
  life_year = discount(cycle_years, disc_cycle),
  qaly = discount(cycle_years * u_pf, disc_cycle)
  )

# Define PD states
state_PD <- heemod::define_state(
  # Costs for the state
  cost_daq_soc = 0,
  cost_daq_new = 0,
  cost_dadmin = 0,
  cost_dman = discount(cman_pd, disc_cycle),  
  cost_ae = 0,
  cost_subs = discount(csubs, disc_cycle),
  cost_total = cost_daq_soc + cost_daq_new + cost_dadmin + cost_dman + cost_ae + cost_subs,
  # Health utility, QALYs and life years
  pf_year = 0,
  life_year = heemod::discount(cycle_years, disc_cycle),
  qaly = heemod::discount(cycle_years * u_pd, disc_cycle)
  )

# Define Death state
state_Death <- heemod::define_state(
  # Costs are zero
  cost_daq_soc = 0,
  cost_daq_new = 0,
  cost_dadmin = 0,
  cost_dman = 0,
  cost_ae = 0,
  cost_subs = 0,
  cost_total = cost_daq_soc + cost_daq_new + cost_dadmin + cost_dman + cost_ae + cost_subs,
  # Health outcomes are zero
  pf_year = 0,
  life_year = 0,
  qaly = 0,
)

# Define strategy for SoC
strat_soc <- heemod::define_strategy(
    transition = psm_soc,
    "PF" = state_PF,
    "PD" = state_PD,
    "Death" = state_Death
  )

# Define strategy for new
strat_new <- heemod::define_strategy(
  transition = psm_new,
  "PF" = state_PF,
  "PD" = state_PD,
  "Death" = state_Death
)

# Create heemod model
heemodel <- heemod::run_model(
  soc = strat_soc,
  new = strat_new,
  parameters = params,
  cycles = Ncycles,
  cost = cost_total,
  effect = qaly,
  init = c(1, 0, 0),
  method = "life-table"
)

## ----dynpricing---------------------------------------------------------------
# Dates
# Date of calculation = 1 September 2025
doc <- lubridate::ymd("20250901")
# Date of LOE for SoC = 1 January 2028
loe_soc_start <- lubridate::ymd("20280101")
# Maturation of SoC prices by LOE + 1 year, i.e. = 1 January 2029
loe_soc_end <- lubridate::ymd("20290101") 
# Date of LOE for new treatment = 1 January 2031
loe_new_start <- lubridate::ymd("20310101")
# Maturation of new treatment prices by LOE + 1 year, i.e. = 1 January 2032
loe_new_end <- lubridate::ymd("20320101") 

# Effect of LoEs on prices once mature
loe_effect_soc <- 0.7
loe_effect_new <- 0.5

# Calculation of weeks since DoC for LoEs and price maturities
wk_start_soc <- floor((loe_soc_start-doc) / lubridate::dweeks(1))
wk_end_soc <- floor((loe_soc_end-doc) / lubridate::dweeks(1))
wk_start_new <- floor((loe_new_start-doc) / lubridate::dweeks(1))
wk_end_new <- floor((loe_new_end-doc) / lubridate::dweeks(1))

# Price maturity times
wk_mature_soc <- wk_end_soc - wk_start_soc
wk_mature_new <- wk_end_new - wk_start_new

# Create a tibble of price indices of length 2T, then pull out columns as needed
# We only need of length T for now, but need of length 2T for future calculations later
pricetib <- tibble(
  model_time = 1:(2*Ncycles),
  model_year = model_time * cycle_years,
  static = 1,
  geninfl = (1 + infl_cycle)^(model_time - 1),
  loef_soc = pmin(pmax(model_time - wk_start_soc, 0), wk_mature_soc) / wk_mature_soc,
  loef_new = pmin(pmax(model_time - wk_start_new, 0), wk_mature_new) / wk_mature_new,
  dyn_soc = geninfl * (1 - loe_effect_soc * loef_soc),
  dyn_new = geninfl * (1 - loe_effect_new * loef_new)
)

# Price indices required for calculations
prices_oth <- pricetib$geninfl
prices_static <- pricetib$static
prices_dyn_soc <- pricetib$dyn_soc
prices_dyn_new <- pricetib$dyn_new

## ----dynuptake----------------------------------------------------------------
# Time for uptake to occur
uptake_years <- 2

# Uptake vector for non-dynamic uptake
uptake_single <- c(1, rep(0, Ncycles-1))

# Uptake vector for dynamic uptake
uptake_weeks <- round(uptake_years / cycle_years)
share_multi <- c((1:uptake_weeks)/uptake_weeks, rep(1, Ncycles-uptake_weeks))
uptake_multi <- rep(1, Ncycles) * share_multi

## ----static1------------------------------------------------------------------
heemodel

## ----static2------------------------------------------------------------------
# Pull out the payoffs of interest from oncpsm
payoffs <- get_dynfields(
    heemodel = heemodel,
    payoffs = c("cost_daq_new", "cost_daq_soc", "cost_total", "qaly", "life_year"),
    discount = "disc"
    ) |>
    mutate(
      model_years = model_time * cycle_years,
      # Derive costs other than drug acquisition, as at time zero
      cost_nondaq = cost_total - cost_daq_new - cost_daq_soc,
      # ... and at the start of each timestep
      cost_nondaq_rup = cost_total_rup - cost_daq_new_rup - cost_daq_soc_rup
    )

# Create and view dataset for SoC
hemout_soc <- payoffs |> filter(int=="soc")
head(hemout_soc)

# Create and view dataset for new intervention
hemout_new <- payoffs |> filter(int=="new")
head(hemout_new)

## ----scen1--------------------------------------------------------------------
# SOC, costs other than drug acquisition
s1_soc_othcost <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_soc$cost_nondaq_rup,
    prices = prices_oth,
    discrate = nomdisc_cycle
    )

# SOC, drug acquisition costs
s1_soc_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_soc$cost_daq_soc_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# SOC, total costs
s1_soc_cost <- s1_soc_daqcost + s1_soc_othcost

# SOC, QALYs
s1_soc_qaly <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_soc$qaly_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# New intervention, costs other than drug acquisition
s1_new_othcost <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_new$cost_nondaq_rup,
    prices = prices_oth,
    discrate = nomdisc_cycle
    )

# New intervention, drug acquisition costs
s1_new_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_new$cost_daq_new_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# New intervention, total costs
s1_new_cost <- s1_new_daqcost + s1_new_othcost

# New intervention, QALYs
s1_new_qaly <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_new$qaly_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# Incremental cost
s1_icost <- s1_new_cost - s1_soc_cost

summary(s1_icost)

# Incremental QALY
s1_iqaly <- s1_new_qaly - s1_soc_qaly

summary(s1_iqaly)

# ICER
s1_icer <- total(s1_icost) / total(s1_iqaly)
s1_icer

## ----calc_scen2---------------------------------------------------------------
# SOC, costs other than drug acquisition are unchanged
s2_soc_othcost <- s1_soc_othcost

# SoC, drug acquisition costs
s2_soc_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_soc$cost_daq_soc_rup,
    prices = prices_dyn_soc,
    discrate = nomdisc_cycle
    )

# SoC, total costs
s2_soc_cost <- s2_soc_daqcost + s2_soc_othcost

# SoC, QALYs are unchanged
s2_soc_qaly <- s1_soc_qaly

# New intervention, costs other than drug acquisition are unchanged
s2_new_othcost <- s1_new_othcost

# New intervention, drug acquisition costs
s2_new_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_new$cost_daq_new_rup,
    prices = prices_dyn_new,
    discrate = nomdisc_cycle
    )

# New intervention, total costs
s2_new_cost <- s2_new_daqcost + s2_new_othcost

# New intervention, QALYs are unchanged
s2_new_qaly <- s1_new_qaly

# Incremental cost
s2_icost <- s2_new_cost - s2_soc_cost

summary(s2_icost)

# Incremental QALY
s2_iqaly <- s2_new_qaly - s2_soc_qaly

summary(s2_iqaly)

# ICER
s2_icer <- total(s2_icost) / total(s2_iqaly)
s2_icer

## ----calc_scen3---------------------------------------------------------------
# SOC, costs other than drug acquisition
s3_soc_othcost <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_soc$cost_nondaq_rup,
    prices = prices_oth,
    discrate = nomdisc_cycle
    )

# SoC, drug acquisition costs
s3_soc_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_soc$cost_daq_soc_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# SoC, total costs
s3_soc_cost <- s3_soc_daqcost + s3_soc_othcost

# SoC, QALYs
s3_soc_qaly <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_soc$qaly_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# New intervention, costs other than drug acquisition
s3_new_othcost <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_new$cost_nondaq_rup,
    prices = prices_oth,
    discrate = nomdisc_cycle
    )

# New intervention, drug acquisition costs
s3_new_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_new$cost_daq_new_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# New intervention, total costs
s3_new_cost <- s3_new_daqcost + s3_new_othcost

# New intervention, QALYs
s3_new_qaly <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_new$qaly_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# Incremental costs
s3_icost <- s3_new_cost - s3_soc_cost

summary(s3_icost)

# Incremental QALYs
s3_iqaly <- s3_new_qaly - s3_soc_qaly

summary(s3_iqaly)

# ICER
s3_icer <- total(s3_icost) / total(s3_iqaly)
s3_icer

## ----calc_scen4---------------------------------------------------------------
# SOC, costs other than drug acquisition are unchanged
s4_soc_othcost <- s3_soc_othcost

# SoC, drug acquisition costs
s4_soc_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_soc$cost_daq_soc_rup,
    prices = prices_dyn_soc,
    discrate = nomdisc_cycle
    )

# SoC, total costs
s4_soc_cost <- s4_soc_daqcost + s4_soc_othcost

# SoC, QALYs are unchanged
s4_soc_qaly <- s3_soc_qaly

# New intervention, costs other than drug acquisition are unchanged
s4_new_othcost <- s3_new_othcost

# New intervention, drug acquisition costs
s4_new_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_new$cost_daq_new_rup,
    prices = prices_dyn_new,
    discrate = nomdisc_cycle
    )

# New intervention, total costs
s4_new_cost <- s4_new_daqcost + s4_new_othcost

# New intervention, QALYs
s4_new_qaly <- s3_new_qaly

# Incremental costs
s4_icost <- s4_new_cost - s4_soc_cost

summary(s4_icost)

# Incremental QALYs
s4_iqaly <- s4_new_qaly - s4_soc_qaly

summary(s4_iqaly)

# ICER
s4_icer <- total(s4_icost) / total(s4_iqaly)
s4_icer

## ----future_calc--------------------------------------------------------------
# Times at which to plot ICER
gtimes <- round((0:(2*thoz))/cycle_years/2)
# SOC drug acquisition costs
gc_soc_daq <- gtimes |>
    purrr::map_vec(\(l) mean(futurepv(
        tzero = l,
        payoffs = hemout_soc$cost_daq_soc_rup,
        prices = prices_dyn_soc,
        discrate = nomdisc_cycle
    ))
)
# SOC other costs
gc_soc_oth <- gtimes |>
    purrr::map_vec(\(l) mean(futurepv(
        tzero = l,
        payoffs = hemout_soc$cost_nondaq_rup,
        prices = prices_oth,
        discrate = nomdisc_cycle
    ))
)
# New drug acquisition costs
gc_new_daq <- gtimes |>
    purrr::map_vec(\(l) mean(futurepv(
        tzero = l,
        payoffs = hemout_new$cost_daq_new_rup,
        prices = prices_dyn_new,
        discrate = nomdisc_cycle
    ))
)
# New other costs
gc_new_oth <- gtimes |>
    purrr::map_vec(\(l) mean(futurepv(
        tzero = l,
        payoffs = hemout_new$cost_nondaq_rup,
        prices = prices_oth,
        discrate = nomdisc_cycle
    ))
)
# Combine in a tibble
ds <- tibble(
    # Time in weeks and years
    time_weeks = gtimes,
    time_years = time_weeks * cycle_years,
    # Evaluation date
    evaldate = doc + time_weeks * 7,
    # Price/inflation index
    pinfl = prices_oth[gtimes + 1],
    # Total costs for each intervention
    totcost_new = gc_new_daq + gc_new_oth,
    totcost_soc = gc_soc_daq + gc_soc_oth,
    # Scenario 1/2 QALYs
    qaly_soc = total(s1_soc_qaly),
    qaly_new = total(s1_new_qaly)
  ) |>
  mutate(
    # Incremental cost and QALYs
    icost = totcost_new-totcost_soc,
    iqaly = qaly_new-qaly_soc,
    # Nominal ICER, and real (inflation adjusted) ICER
    Nominal = icost/iqaly,
    Real = Nominal / pinfl
  ) |>
  # Pivot to long so can be used in a graphic
  pivot_longer(
    cols = c("Nominal", "Real"),
    names_to = "Type",
    values_to = "ICER"
  )

## ----calc_future--------------------------------------------------------------
# Plot real and nominal present value over time
ggplot(ds,
  aes(x = evaldate, y = ICER, color=Type)) +
  geom_line() +
  labs(x = "Evaluation date") +
  geom_hline(yintercept = ds$ICER[1], linetype='dotted') +
  geom_vline(xintercept = loe_new_start, linetype='dashed') +
  geom_vline(xintercept = loe_soc_start, linetype='dashed') +
  scale_y_continuous(
    labels = scales::comma,
    limits=c(0, 150000)
    )

#  Copyright (c) 2025 Merck & Co., Inc., Rahway, NJ, USA and its affiliates.
#  All rights reserved.
#
#  This file is part of the dynamicpv program.
#
#  dynamicpv is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#' Present values with dynamic pricing and dynamic uptake
#' 
#' Calculate present value for a payoff with dynamic (lifecycle) pricing and dynamic uptake (stacked cohorts).
#'
#' Suppose payoffs in relation to patients receiving treatment (such as costs or health outcomes) occur over timesteps \eqn{t=1, ..., T}. Let us partition time as follows.
#' 
#' - Suppose \eqn{j=1,...,T} indexes the time at which the patient begins treatment.
#' - Suppose \eqn{k=1,...,T} indexes time since initiating treatment.
#' 
#' In general, \eqn{t=j+k-1}, and we are interested in the set of \eqn{(j,k)} such that \eqn{1 \leq t \leq T}.
#' 
#' For example, \eqn{t=3} comprises:
#' 
#' - patients who are in the third timestep of treatment that began in timestep 1: (j,k)=(1,3);
#' - patients who are in the second timestep of treatment that began in timestep 2, (j,k)=(2,2); and
#' - patients who are in the first timestep of treatment that began in timestep 3, (j,k)=(3,1)
#' 
#' The [Present Value](https://en.wikipedia.org/wiki/Present_value) of a cashflow \eqn{p_k} for the \eqn{u_j} patients who began treatment at time \eqn{j} and who are in their \eqn{k}th timestep of treatment is as follows
#' \deqn{PV(j,k,l) = u_j \cdot p_k \cdot R_{j+k+l-1} \cdot (1+i)^{2-j-k}}
#' where \eqn{i} is the risk-free discount rate per timestep, \eqn{p_k} is the cashflow amount in today’s money, and \eqn{p_k \cdot R_{j+k+l-1}} is the nominal amount of the cashflow at the time it is incurred, allowing for an offset of \eqn{l = tzero}.
#' 
#' The total present value, \eqn{TPV(l)}, is therefore the sum over all \eqn{j} and \eqn{k} within the time horizon \eqn{T}, namely:
#' \deqn{TPV(l) = \sum_{j=1}^{T} \sum_{k=1}^{T-j+1} PV(j,k, l) \\
#' \;
#' = \sum_{j=1}^{T} \sum_{k=1}^{T-j+1} u_j \cdot p_k \cdot R_{l+j+k-1} \cdot (1+i)^{2-j-k}}
#' 
#' This function calculates \eqn{PV(j,k,l)} for all values of \eqn{j}, \eqn{k} and \eqn{l}, and returns this in a tibble.
#' @param uptakes Vector of patient uptake over time
#' @param horizon Time horizon for the calculation (length must be less than or equal to the length of payoffs)
#' @param tzero Time at the date of calculation, to be used in lookup in prices vector
#' @param payoffs Vector of payoffs of interest (numeric vector)
#' @param prices Vector of price indices through the time horizon of interest
#' @param discrate Discount rate per timestep, corresponding to price index
#' @returns A tibble of class "dynpv" with the following columns:
#' - `j`: Time at which patients began treatment
#' - `k`: Time since patients began treatment
#' - `l`: Time offset for the price index (from `tzero`)
#' - `t`: Equals \eqn{j+k-1}
#' - `uj`: Uptake of patients beginning treatment at time \eqn{j} (from `uptakes`)
#' - `pk`: Cashflow amount in today's money in respect of patients at time \eqn{k} since starting treatment (from `payoffs`)
#' - `R`: Index of prices over time \eqn{l+t} (from `prices`)
#' - `v`: Discounting factors, \eqn{(1+i)^{1-t}}, where `i` is the discount rate per timestep
#' - `pv`: Present value, \eqn{PV(j,k,l)}
#' @export
#' @examples
#' # Simple example
#' pv1 <- dynpv(
#'    uptakes = (1:10), # 1 patient uptakes in timestep 1, 2 patients in timestep 2, etc
#'    tzero = c(0,5), # Calculations are performed with prices at times 0 and 5
#'    payoffs = 90 + 10*(1:10), # Payoff vector of length 10 = (100, 110, ..., 190)
#'    prices = 1.02^((1:15)-1), # Prices increase at 2\% per timestep in future
#'    discrate = 0.05 # The nominal discount rate is 5\% per timestep;
#'                    # the real discount rate per timestep is 3\% (=5\% - 3\%)
#' )
#' pv1
#' summary(pv1)
#' 
#' # More complex example, using cashflow output from a heemod model
#'
#' # Obtain dataset
#' democe <- get_dynfields(
#'    heemodel = oncpsm,
#'    payoffs = c("cost_daq_new", "cost_total", "qaly"),
#'    discount = "disc"
#'    )
#'
#' # Obtain short payoff vector of interest
#' payoffs <- democe |>
#'    dplyr::filter(int=="new", model_time<11) |>
#'    dplyr::mutate(cost_oth = cost_total - cost_daq_new)
#'
#' # Example calculation
#' pv2 <- dynpv(
#'    uptakes = rep(1, nrow(payoffs)),
#'    payoffs = payoffs$cost_oth,
#'    prices = 1.05^(0:(nrow(payoffs)-1)),
#'    discrate = 0.08
#' )
#' summary(pv2)
dynpv <- function(
    uptakes = 1,
    payoffs,
    horizon = length(payoffs),
    tzero = 0,
    prices = rep(1, length(payoffs)+tzero),
    discrate = 0
    ){
  # Avoid no visible binding note
  j <- k <- l <- uj <- pk <- R <- v <- NULL
  # Trim
  uptakes <- trim_vec(uptakes)
  payoffs <- trim_vec(payoffs)
  # Create a dataset for each combination of time
  df <- expand_grid(j=1:length(uptakes), k=1:length(payoffs), l=tzero) |>
    mutate(t= j + k - 1) |>
    # Remove time entries that are outside the time horizon
    filter(t <= horizon) |>
    mutate(
      uj = uptakes[j],
      pk = payoffs[k],
      R = prices[l + t],
      v = (1+discrate)^(1 - t),
      pv = uj * pk * R * v
    )
  class(df) <- c("dynpv", class(df))
  return(df)
}

#' Trim the tailing zeroes from a long vector
#'
#' @param vec Vector. Final elements may be zero.
#' @returns A vector whose length is shorter than the original, if there were trailing zero elements
#' @export
#' @examples
#' trim_vec(c(1:10, rep(0,3)))
trim_vec <- function(vec){
  # Identify the last element before vector elements are zero
  trimto <- which(rev(cumsum(rev(vec)))==vec)[1]
  # Return trimmed vector
  return(vec[1:trimto])
}

#  Copyright (c) 2025 Merck & Co., Inc., Rahway, NJ, USA and its affiliates.
#  All rights reserved.
#
#  This file is part of the dynamicpv program.
#
#  dynamicpv is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

## Collection of functions to extract payoffs from other systems, e.g. heemod
## ==========================================================================

#' Helper function to get a tibble of the relevant fields from heemod output
#' @param heemodel A health economic model object from the *heemod* package (see [heemod::heemod-package]).
#' @param payoffs Field names of payoffs of interest (string vector)
#' @param discount Name of parameter providing discount rate per cycle (string)
#' @param fname Export data to a .CSV file of this name, if given (character)
#' @returns Tibble of payoffs taken from the heemod model, by intervention and model timestep (`model_time`).
#'
#' The field `vt` is calculated as `(1+i)^(1-model_time)`, where `i` is the discount rate per model timestep set in the *heemod* model through the parameter `disc_cycle`. This can be useful in 'rolling-up' payoff values to the timestep in which they were incurred.
#'
#' An additional set of payoffs (identified with a "_rup" suffix) provides calculations of the payoffs as at the start of the timestep in which they were incurred, i.e. original payoff / `vt`.
#' @seealso [heemod::heemod-package]
#' @export
#' @examples
#' democe <- get_dynfields(
#'    heemodel = oncpsm,
#'    payoffs = c("cost_daq_new", "cost_total", "qaly"),
#'    discount = "disc"
#'    )
#' head(democe)
get_dynfields <- function(heemodel, payoffs, discount, fname=NA){
  # Set certain variables to NULL to avoid 'no visible binding' note
  model_time <- vt_rup <- NULL
  # Pull out interventions
  int <- names(heemodel$eval_strategy_list)
  # Pull out discount rate
  discrate <- get_param_value(heemodel, discount)
  # Create a list to store data for each intervention
  ds <- vector(mode = "list", length = length(int))
  # Pull in tibble into list for each intervention
  for (i in seq(int)){
    ds[[i]] <- heemodel$eval_strategy_list[[int[i]]]$values |>
      as_tibble() |>
      select(c("model_time", all_of(payoffs))) |>
      mutate(int = int[i])
  }
  # Unnest
  ds <- bind_rows(ds) |>
    # Add discounting variable
    mutate(vt = (1 + discrate)^(1 - model_time))
  # Create rolled-up variables, with "r" prefix to their name
  rs <- ds |>
    mutate(across(all_of(payoffs), ~.x/vt))
  # Join the rolled-up data to the original data
  ds <- ds |>
    left_join(rs, by=c("model_time", "int"), suffix=c("", "_rup")) |>
    select(-vt_rup)
  # Export as CSV
  if (!is.na(fname)) {readr::write_csv(ds, file=paste0(fname, ".csv"))}
  # Return
  return(ds)
}

#' Obtain parameter value(s) from a heemod output
#' @inheritParams get_dynfields
#' @param param Name of parameter to extract from the heemod model
#' @returns Value of the parameter from the heemod model
#' @seealso [heemod::heemod-package]
#' @export
#' @examples
#' get_param_value(
#'    heemodel = oncpsm,
#'    param = "disc"
#'    )
get_param_value <- function(heemodel, param){
  rlang::eval_tidy(heemodel$parameters[[param]])
}

# Test the future calculations

# Get data
democe <- get_dynfields(
   heemodel = oncpsm,
   payoffs = c("cost_daq_new", "cost_total", "qaly"),
   discount = "disc"
   )

# Get discount rate
discrate <- get_param_value(oncpsm, "disc")

# Obtain payoff vector of interest
payoffs <- democe |>
   dplyr::filter(int=="new") |>
   dplyr::mutate(cost_oth = cost_total_rup - cost_daq_new_rup)
Nt <- nrow(payoffs)

# Simple calculation - no discounting
test_that("Simple calculation works", {
  # Actual
  act_val1 <- futurepv(
      tzero = 0:9,
      payoffs = payoffs$cost_oth,
      prices = rep(1, 2*Nt),
      discrate = discrate
      )
  # Expected
  exp_val1 <- rep(sum(payoffs$cost_total - payoffs$cost_daq_new), 10)
  # Test
  expect_equal(mean(act_val1)[[2]], exp_val1)
})

# More complex calculations - different time horizon
test_that("Complex calculation works", {
  # Actual
  act_val2 <- futurepv(
    tzero = 0,
    payoffs = payoffs$cost_oth[1:(Nt/2)],
    prices = rep(1, 2*Nt),
    discrate = discrate
  )
  # Expected
  exp_val2 <- sum((payoffs$cost_total - payoffs$cost_daq_new)[1:(Nt/2)])
  # Tests
  expect_equal(mean(act_val2), exp_val2)
})

# 10. No error when running print
test_that("No error when applying print summary", {
  # Some sample dynpv values
  fpv1 <- futurepv(
    tzero = 0,
    payoffs = payoffs$cost_oth[1:(Nt/2)],
    prices = rep(1, 2*Nt),
    discrate = discrate
  )
  fpv2 <- futurepv(
      tzero = 0:9,
      payoffs = payoffs$cost_oth,
      prices = rep(1, 2*Nt),
      discrate = discrate
      )
  # Check no error
  expect_no_error(print(summary(fpv1)))
  expect_no_error(print(summary(fpv2)))
})

# Test the dynamic calculations

# Get data
democe <- get_dynfields(
  heemodel = oncpsm,
  payoffs = c("cost_daq_new", "cost_total", "qaly"),
  discount = "disc"
  )

# Get discount rate
discrate <- get_param_value(oncpsm, "disc")

# Obtain payoff vector of interest
payoffs <- democe |>
  dplyr::filter(int=="new") |>
  dplyr::mutate(cost_oth = cost_total_rup - cost_daq_new_rup)
Nt <- nrow(payoffs)

# 1. Simple calculation - one off uptake, no price escalation, no discounting
test_that("Simple calculation", {
  # Actual
  act_dval1 <- dynpv(
    uptakes = c(1, rep(0, Nt-1)),
    payoffs = payoffs$cost_oth,
    prices = rep(1, Nt),
    discrate = discrate
  )
  # Expected
  exp_dval1 <- sum(payoffs$cost_total - payoffs$cost_daq_new)
  # Test total and mean
  expect_equal(total(act_dval1), exp_dval1)
  expect_equal(mean(act_dval1), exp_dval1)
})

# 2. As (1) but with 5 x the uptake
test_that("5x the uptake of (1)", {
  # Actual
  act_dval2 <- dynpv(
    uptakes = c(5, rep(0, Nt-1)),
    payoffs = payoffs$cost_oth,
    prices = rep(1, Nt),
    discrate = discrate
    )
  # Expected
  exp_dval2 <- 5 * sum(payoffs$cost_total - payoffs$cost_daq_new)
  # Test total and mean
  expect_equal(total(act_dval2), exp_dval2)
  expect_equal(mean(act_dval2), exp_dval2/5)
})

# 3. As (1) but with discounting at 1.5%
test_that("As (1) but 1.5% discounting", {
  # Actual
  act_dval3 <- dynpv(
    uptakes = c(1, rep(0, Nt-1)),
    payoffs = payoffs$cost_oth,
    prices = rep(1, Nt),
    discrate = 0.015
    )
  # Expect
  vt <- (1.015)^(1-(1:Nt))
  exp_dval3 <- sum(vt * (payoffs$cost_total_rup - payoffs$cost_daq_new_rup))
  # Test total and mean
  expect_equal(total(act_dval3), exp_dval3)
  expect_equal(mean(act_dval3), exp_dval3)
})

# 4. As (3) but with 1% increasing price index
test_that("As (1) but 1% price increases", {
  # Actual
  prices <- 1.01^((1:Nt)-1)
  act_dval4 <- dynpv(
    uptakes = c(1, rep(0, Nt-1)),
    payoffs = payoffs$cost_oth,
    prices = prices,
    discrate = 0.015
    )
  # Expected
  vt <- (1.015)^(1-(1:Nt))
  exp_dval4 <- sum(vt* prices * (payoffs$cost_total_rup - payoffs$cost_daq_new_rup))
  # Test total and mean
  expect_equal(total(act_dval4), exp_dval4)
  expect_equal(mean(act_dval4), exp_dval4)
})

# 5. As (1) but with flat incidence
test_that("As (1) but flat incidence", {
  # Actual
  act_dval5 <- dynpv(
    uptakes = rep(1, Nt),
    payoffs = payoffs$cost_oth,
    prices = rep(1, Nt),
    discrate = 0
    )
  # Expected
  exp_dval5 <- sum((Nt:1) * payoffs$cost_oth)
  # Test total and mean
  expect_equal(total(act_dval5), exp_dval5)
  expect_equal(mean(act_dval5), exp_dval5/Nt)
})

# 6. As (5) but with price increases
test_that("As (5) but with price increases", {
  # Actual
  prices <- 1.01^((1:Nt)-1)
  act_dval6 <- dynpv(
    uptakes = rep(1, Nt),
    payoffs = payoffs$cost_oth,
    prices = prices,
    discrate = discrate
    )
  # Expected lower bound - exp_dval5
  exp_dval6 <- sum((Nt:1) * payoffs$cost_oth)
  # Test mean only
  expect_gt(mean(act_dval6), exp_dval6)
})


# 7. As (6) but with multiple time points
test_that("As (6) but with multiple time points (tzero)", {
  # Actual
  prices <- 1.01^((1:(2*Nt))-1)
  tzero <- (0:4)*52
  act_dval6 <- dynpv(
    uptakes = rep(1, Nt),
    payoffs = payoffs$cost_oth,
    prices = prices,
    tzero = tzero,
    discrate = discrate
  )
  # Expected
  exp_dval6_0 <- dynpv(
    uptakes = rep(1, Nt),
    payoffs = payoffs$cost_oth,
    prices = prices,
    tzero = 0,
    discrate = discrate
  )
  exp_dval6_1 <- dynpv(
    uptakes = rep(1, Nt),
    payoffs = payoffs$cost_oth,
    prices = prices,
    tzero = 1*52,
    discrate = discrate
  )
  exp_dval6_2 <- dynpv(
    uptakes = rep(1, Nt),
    payoffs = payoffs$cost_oth,
    prices = prices,
    tzero = 2*52,
    discrate = discrate
  )
  exp_dval6_3 <- dynpv(
    uptakes = rep(1, Nt),
    payoffs = payoffs$cost_oth,
    prices = prices,
    tzero = 3*52,
    discrate = discrate
  )
  exp_dval6_4 <- dynpv(
    uptakes = rep(1, Nt),
    payoffs = payoffs$cost_oth,
    prices = prices,
    tzero = 4*52,
    discrate = discrate
  )
  # Test total and mean
  expect_equal(
    total(act_dval6)$total,
    c(total(exp_dval6_0), total(exp_dval6_1), total(exp_dval6_2),
      total(exp_dval6_3), total(exp_dval6_4))
  )
  expect_equal(
    mean(act_dval6)$mean,
    c(mean(exp_dval6_0), mean(exp_dval6_1), mean(exp_dval6_2),
      mean(exp_dval6_3), mean(exp_dval6_4)))
})


# 8. Test addition
test_that("Addition of two dynamic pv's", {
  # Actual
  act_dval8a <- dynpv(
    uptakes = c(1, rep(0, Nt-1)),
    payoffs = payoffs$cost_oth * (1:Nt),
    prices = rep(1, Nt),
    discrate = 0
    )
  act_dval8b <- dynpv(
    uptakes = c(1, rep(0, Nt-1)),
    payoffs = payoffs$cost_oth,
    prices = rep(1, Nt),
    discrate = 0
    )
  act_add <- act_dval8a + act_dval8b
  # Expected
  exp_dval8a <- sum(payoffs$cost_oth * (1:Nt))
  exp_dval8b <- sum(payoffs$cost_oth)
  # Test total and mean
  expect_equal(total(act_dval8a), exp_dval8a)
  expect_equal(total(act_dval8b), exp_dval8b)
  expect_equal(total(act_add), exp_dval8a + exp_dval8b)
  expect_equal(uptake(act_dval8a) + uptake(act_dval8b), 2)
})

# 9. Test subtraction of two pv's
test_that("Subtraction of two pv's", {
  # Actual
  act_dval8a <- dynpv(
    uptakes = c(2, rep(0, Nt-1)),
    payoffs = payoffs$cost_oth * (1:Nt),
    prices = rep(1, Nt),
    discrate = 0
    )
  act_dval8b <- dynpv(
    uptakes = c(1, rep(0, Nt-1)),
    payoffs = payoffs$cost_oth,
    prices = rep(1, Nt),
    discrate = 0
    )
  act_sub <- act_dval8a - act_dval8b
  # Expected
  exp_dval8a <- 2 * sum(payoffs$cost_oth * (1:Nt))
  exp_dval8b <- sum(payoffs$cost_oth)
  # Test total
  expect_equal(total(act_dval8a), exp_dval8a)
  expect_equal(total(act_dval8b), exp_dval8b)
  expect_equal(total(act_sub), exp_dval8a - exp_dval8b)
  expect_equal(uptake(act_dval8a) - uptake(act_dval8b), 2-1)
})

# 10. No error when running print
test_that("No error when applying print summary", {
  # Some sample dynpv values
  prices <- 1.01^((1:(2*Nt))-1)
  act_dval4 <- dynpv(
    uptakes = c(1, rep(0, Nt-1)),
    payoffs = payoffs$cost_oth,
    prices = prices,
    discrate = 0.015
  )
  tzero <- (0:4)*52
  act_dval6 <- dynpv(
    uptakes = rep(1, Nt),
    payoffs = payoffs$cost_oth,
    prices = prices,
    tzero = tzero,
    discrate = discrate
  )
  act_dval8a <- dynpv(
    uptakes = c(2, rep(0, Nt-1)),
    payoffs = payoffs$cost_oth * (1:Nt),
    prices = rep(1, Nt),
    discrate = 0
  )
  # Check no error
  expect_no_error(print(summary(act_dval4)))
  expect_no_error(print(summary(act_dval8a)))
  expect_no_error(print(summary(act_dval4 + act_dval8a)))
  expect_no_error(print(summary(act_dval4 - act_dval8a)))
  expect_no_error(print(summary(act_dval6)))
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

# Numeric notation in this document
# With thanks to https://stackoverflow.com/questions/18965637/set-global-thousand-separator-on-knitr/18967590#18967590
knitr::knit_hooks$set(
    inline = function(x) {
        if(!is.numeric(x)){
            x
        } else if(x<100) {
            prettyNum(round(x, 3), big.mark=",")
        } else {
            prettyNum(round(x, 0), big.mark=",")
        }
    }
)

## ----setup, echo=TRUE, message=FALSE------------------------------------------
library(dplyr)
library(lubridate)
library(heemod)
library(dynamicpv)

## ----constants, include=FALSE-------------------------------------------------
# Time constants
days_in_year <- 365.25
days_in_week <- 7
cycle_years <- days_in_week / days_in_year # Duration of a one week cycle in years

# Time horizon (years) and number of cycles
thoz <- 20
Ncycles <- ceiling(thoz/cycle_years)

# Real discounting
disc_year <- 0.03 # Per year
disc_cycle <- (1+disc_year)^(cycle_years) - 1 # Per cycle

# Price inflation
infl_year <- 0.025 # Per year
infl_cycle <- (1+infl_year)^(cycle_years) - 1 # Per cycle

# Nominal discounting
nomdisc_year <- (1+disc_year)*(1+infl_year) - 1
nomdisc_cycle <- (1+nomdisc_year)^(cycle_years) - 1 # Per cycle

## ----heemod, include=FALSE----------------------------------------------------
# State names
state_names = c(
  progression_free = "PF",
  progression = "PD",
  death = "Death"
  )

# PFS distribution for SoC with Exp() distribution and mean of 50 weeks
surv_pfs_soc <- heemod::define_surv_dist(
  distribution = "exp",
  rate = 1/50
)

# OS distribution for SoC with Lognorm() distribution, meanlog = 4.5, sdlog = 1
# This implies a mean of exp(4 + 0.5 * 1^2) = exp(4.5) = 90 weeks
surv_os_soc <- heemod::define_surv_dist(
  distribution = "lnorm",
  meanlog = 4,
  sdlog = 1
)

# PFS and OS distributions for new
surv_pfs_new <- heemod::apply_hr(surv_pfs_soc, hr=0.5)
surv_os_new <- heemod::apply_hr(surv_os_soc, hr=0.6)

# Define partitioned survival model, soc
psm_soc <- heemod::define_part_surv(
  pfs = surv_pfs_soc,
  os = surv_os_soc,
  terminal_state = FALSE,
  state_names = state_names
  )

# Define partitioned survival model, soc
psm_new <- heemod::define_part_surv(
  pfs = surv_pfs_new,
  os = surv_os_new,
  terminal_state = FALSE,
  state_names = state_names
  )

# Parameters
params <- heemod::define_parameters(
  # Discount rate
  disc = disc_cycle,
  # Disease management costs
  cman_pf = 80,
  cman_pd = 20,
  # Drug acquisition costs - the SoC regime only uses SoC drug, the New regime only uses New drug
  cdaq_soc = dispatch_strategy(
    soc = 400,
    new = 0
  ),
  cdaq_new = dispatch_strategy(
    soc = 0,
    new = 1500
  ),
  # Drug administration costs
  cadmin = dispatch_strategy(
    soc = 50,
    new = 75
  ),
  # Adverse event risks
  risk_ae = dispatch_strategy(
    soc = 0.08,
    new = 0.1
  ),
  # Adverse event average costs
  uc_ae = 2000,
  # Subsequent treatments
  csubs = dispatch_strategy(
    soc = 1200,
    new = 300
  ),
  # Health state utilities
  u_pf = 0.8,
  u_pd = 0.6,
)

# Define PF states
state_PF <- heemod::define_state(
  # Costs for the state
  cost_daq_soc = discount(cdaq_soc, disc_cycle),
  cost_daq_new = discount(cdaq_new, disc_cycle),
  cost_dadmin = discount(cadmin, disc_cycle),
  cost_dman = discount(cman_pf, disc_cycle),  
  cost_ae = risk_ae * uc_ae,
  cost_subs = 0,
  cost_total = cost_daq_soc + cost_daq_new + cost_dadmin + cost_dman + cost_ae + cost_subs,
  # Health utility, QALYs and life years
  pf_year = discount(cycle_years, disc_cycle),
  life_year = discount(cycle_years, disc_cycle),
  qaly = discount(cycle_years * u_pf, disc_cycle)
  )

# Define PD states
state_PD <- heemod::define_state(
  # Costs for the state
  cost_daq_soc = 0,
  cost_daq_new = 0,
  cost_dadmin = 0,
  cost_dman = discount(cman_pd, disc_cycle),  
  cost_ae = 0,
  cost_subs = discount(csubs, disc_cycle),
  cost_total = cost_daq_soc + cost_daq_new + cost_dadmin + cost_dman + cost_ae + cost_subs,
  # Health utility, QALYs and life years
  pf_year = 0,
  life_year = heemod::discount(cycle_years, disc_cycle),
  qaly = heemod::discount(cycle_years * u_pd, disc_cycle)
  )

# Define Death state
state_Death <- heemod::define_state(
  # Costs are zero
  cost_daq_soc = 0,
  cost_daq_new = 0,
  cost_dadmin = 0,
  cost_dman = 0,
  cost_ae = 0,
  cost_subs = 0,
  cost_total = cost_daq_soc + cost_daq_new + cost_dadmin + cost_dman + cost_ae + cost_subs,
  # Health outcomes are zero
  pf_year = 0,
  life_year = 0,
  qaly = 0,
)

# Define strategy for SoC
strat_soc <- heemod::define_strategy(
    transition = psm_soc,
    "PF" = state_PF,
    "PD" = state_PD,
    "Death" = state_Death
  )

# Define strategy for new
strat_new <- heemod::define_strategy(
  transition = psm_new,
  "PF" = state_PF,
  "PD" = state_PD,
  "Death" = state_Death
)

# Create heemod model
heemodel <- heemod::run_model(
  soc = strat_soc,
  new = strat_new,
  parameters = params,
  cycles = Ncycles,
  cost = cost_total,
  effect = qaly,
  init = c(1, 0, 0),
  method = "life-table"
)

## ----dynpricing, include=FALSE------------------------------------------------
# Dates
# Date of calculation = 1 September 2025
doc <- lubridate::ymd("20250901")
# Date of LOE for SoC = 1 January 2028
loe_soc_start <- lubridate::ymd("20280101")
# Maturation of SoC prices by LOE + 1 year, i.e. = 1 January 2029
loe_soc_end <- lubridate::ymd("20290101") 
# Date of LOE for new treatment = 1 January 2031
loe_new_start <- lubridate::ymd("20310101")
# Maturation of new treatment prices by LOE + 1 year, i.e. = 1 January 2032
loe_new_end <- lubridate::ymd("20320101") 

# Effect of LoEs on prices once mature
loe_effect_soc <- 0.7
loe_effect_new <- 0.5

# Calculation of weeks since DoC for LoEs and price maturities
wk_start_soc <- floor((loe_soc_start-doc) / lubridate::dweeks(1))
wk_end_soc <- floor((loe_soc_end-doc) / lubridate::dweeks(1))
wk_start_new <- floor((loe_new_start-doc) / lubridate::dweeks(1))
wk_end_new <- floor((loe_new_end-doc) / lubridate::dweeks(1))

# Price maturity times
wk_mature_soc <- wk_end_soc - wk_start_soc
wk_mature_new <- wk_end_new - wk_start_new

# Create a tibble of price indices of length 2T, then pull out columns as needed
# We only need of length T for now, but need of length 2T for future calculations later
pricetib <- dplyr::tibble(
  model_time = 1:(2*Ncycles),
  model_year = model_time * cycle_years,
  static = 1,
  geninfl = (1 + infl_cycle)^(model_time - 1),
  loef_soc = pmin(pmax(model_time - wk_start_soc, 0), wk_mature_soc) / wk_mature_soc,
  loef_new = pmin(pmax(model_time - wk_start_new, 0), wk_mature_new) / wk_mature_new,
  dyn_soc = geninfl * (1 - loe_effect_soc * loef_soc),
  dyn_new = geninfl * (1 - loe_effect_new * loef_new)
)

# Price indices required for calculations
prices_oth <- pricetib$geninfl
prices_static <- pricetib$static
prices_dyn_soc <- pricetib$dyn_soc
prices_dyn_new <- pricetib$dyn_new

## ----dynuptake, include=FALSE-------------------------------------------------
# Time for uptake to occur
uptake_years <- 2

# Uptake vector for non-dynamic uptake
uptake_single <- c(1, rep(0, Ncycles-1))

# Uptake vector for dynamic uptake
uptake_weeks <- round(uptake_years / cycle_years)
share_multi <- c((1:uptake_weeks)/uptake_weeks, rep(1, Ncycles-uptake_weeks))
uptake_multi <- rep(1, Ncycles) * share_multi

## ----static2, include=FALSE---------------------------------------------------
# Pull out the payoffs of interest from oncpsm
payoffs <- get_dynfields(
    heemodel = heemodel,
    payoffs = c("cost_daq_new", "cost_daq_soc", "cost_total", "qaly", "life_year"),
    discount = "disc"
    ) |>
    dplyr::mutate(
      model_years = model_time * cycle_years,
      # Derive costs other than drug acquisition, as at time zero
      cost_nondaq = cost_total - cost_daq_new - cost_daq_soc,
      # ... and at the start of each timestep
      cost_nondaq_rup = cost_total_rup - cost_daq_new_rup - cost_daq_soc_rup
    )

# Create and view dataset for SoC
hemout_soc <- payoffs |> dplyr::filter(int=="soc")
head(hemout_soc)

# Create and view dataset for new intervention
hemout_new <- payoffs |> dplyr::filter(int=="new")
head(hemout_new)

## ----bimshare-----------------------------------------------------------------
# BIM settings
bi_horizon_yrs <- 5
bi_horizon_wks <- round(bi_horizon_yrs / cycle_years)
bi_discount <- 0

# Newly eligible patients
newly_eligible <- rep(1, Ncycles)

# Time for uptake to occur
uptake_years <- 2
uptake_weeks <- round(uptake_years / cycle_years)

# Market share of new intervention
share_multi <- c((1:uptake_weeks)/uptake_weeks, rep(1, Ncycles-uptake_weeks))

# Newly eligible patients receiving each intervention, "world with"
uptake_new <- newly_eligible * share_multi
uptake_soc <- newly_eligible - uptake_new

## ----bim_wout1----------------------------------------------------------------
# World without new intervention

# SoC, drug acquisition costs
wout1_soc_daqcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_soc$cost_daq_soc_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# SoC, other costs
wout1_soc_othcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_soc$cost_nondaq_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# Total budgetary costs
budget_wout1_soc <- wout1_soc_daqcost + wout1_soc_othcost
budget_wout1_new <- 0
budget_wout1 <- budget_wout1_soc

## ----bim_with1----------------------------------------------------------------
# World with: SoC are zero

# New intervention, drug acquisition costs
with1_new_daqcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_new$cost_daq_new_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# New intervention, other costs
with1_new_othcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_new$cost_nondaq_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# Total
budget_with1_soc <- 0
budget_with1_new <- with1_new_daqcost + with1_new_othcost
budget_with1 <- budget_with1_new

## ----bim_iwith1---------------------------------------------------------------
# Budget impact
bi1 <- budget_with1 - budget_wout1
summary(bi1)

## ----bim_wout2----------------------------------------------------------------
# World without new intervention

# SoC, drug acquisition costs
wout2_soc_daqcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_soc$cost_daq_soc_rup,
    horizon = bi_horizon_wks,
    prices = prices_dyn_soc,
    discrate = bi_discount
    )

# SoC, other costs
wout2_soc_othcost <- wout1_soc_othcost

# Total budgetary costs
budget_wout2_soc <- wout2_soc_daqcost + wout2_soc_othcost
budget_wout2_new <- 0
budget_wout2 <- budget_wout2_soc

## ----bim_with2----------------------------------------------------------------
# World with: SoC are zero

# New intervention, drug acquisition costs
with2_new_daqcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_new$cost_daq_new_rup,
    horizon = bi_horizon_wks,
    prices = prices_dyn_new,
    discrate = bi_discount
    )

# New intervention, other costs
with2_new_othcost <- with1_new_othcost

# Total
budget_with2_soc <- 0
budget_with2_new <- with2_new_daqcost + with2_new_othcost
budget_with2 <- budget_with2_new

## ----bim_iwith2---------------------------------------------------------------
# Budget impact
bi2 <- budget_with2 - budget_wout2
summary(bi2)

## ----bim_wout3----------------------------------------------------------------
# World without new intervention
# SoC, drug acquisition costs
wout3_soc_daqcost <- wout1_soc_daqcost
# SoC, other costs
wout3_soc_othcost <- wout1_soc_othcost
# Total budgetary costs
budget_wout3 <- budget_wout3_soc <- wout3_soc_daqcost + wout3_soc_othcost

## ----bim_with3----------------------------------------------------------------
# World with

# SoC, drug acquisition costs
with3_soc_daqcost <- dynpv(
    uptakes = uptake_soc,
    payoffs = hemout_soc$cost_daq_soc_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# SoC, other costs
with3_soc_othcost <- dynpv(
    uptakes = uptake_soc,
    payoffs = hemout_soc$cost_nondaq_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# New intervention, drug acquisition costs
with3_new_daqcost <- dynpv(
    uptakes = uptake_new,
    payoffs = hemout_new$cost_daq_new_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# New intervention, other costs
with3_new_othcost <- dynpv(
    uptakes = uptake_new,
    payoffs = hemout_new$cost_nondaq_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# Total
budget_with3_soc <- with3_soc_daqcost + with3_soc_othcost
budget_with3_new <- with3_new_daqcost + with3_new_othcost
budget_with3 <- budget_with3_soc + budget_with3_new

## ----warnlength---------------------------------------------------------------
# The uptake vector for the new intervention is long
length(trim_vec(uptake_new))

# The uptake vector for the SoC is short, once trimmed of excess zeros
length(trim_vec(uptake_soc))

## ----bim_iwith3---------------------------------------------------------------
# Budget impact
bi3_soc <- budget_with3_soc - budget_wout3_soc
bi3_new <- budget_with3_new
bi3 <- budget_with3 - budget_wout3

summary(bi3)

## ----bim_wout4----------------------------------------------------------------
# World without new intervention

# SoC, drug acquisition costs
wout4_soc_daqcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_soc$cost_daq_soc_rup,
    horizon = bi_horizon_wks,
    prices = prices_dyn_soc,
    discrate = bi_discount
    )

# SoC, other costs - unchanged from static calculations
wout4_soc_othcost <- wout3_soc_othcost

# Total budgetary costs
budget_wout4 <- budget_wout4_soc <- wout4_soc_daqcost + wout4_soc_othcost

## ----bim_with4----------------------------------------------------------------
# World with

# SoC, drug acquisition costs
with4_soc_daqcost <- dynpv(
    uptakes = uptake_soc,
    payoffs = hemout_soc$cost_daq_soc_rup,
    horizon = bi_horizon_wks,
    prices = prices_dyn_soc,
    discrate = bi_discount
    )

# SoC, other costs
with4_soc_othcost <- with3_soc_othcost

# New intervention, drug acquisition costs
with4_new_daqcost <- dynpv(
    uptakes = uptake_new,
    payoffs = hemout_new$cost_daq_new_rup,
    horizon = bi_horizon_wks,
    prices = prices_dyn_new,
    discrate = bi_discount
    )

# New intervention, other costs
with4_new_othcost <- with3_new_othcost

# Total
budget_with4_soc <- with4_soc_daqcost + with4_soc_othcost
budget_with4_new <- with4_new_daqcost + with4_new_othcost
budget_with4 <- budget_with4_soc + budget_with4_new

## ----bim_iwith4---------------------------------------------------------------
# Budget impact
bi4_soc <- budget_with4_soc - budget_wout4_soc
bi4_new <- budget_with4_new
bi4 <- budget_with4 - budget_wout4

summary(bi4)

#  Copyright (c) 2025 Merck & Co., Inc., Rahway, NJ, USA and its affiliates.
#  All rights reserved.
#
#  This file is part of the dynamicpv program.
#
#  dynamicpv is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

## Convenience function to calculate 'future' cost-effectiveness results
## ========================================================================

#' Calculate present value for a payoff in a single cohort with dynamic pricing across multiple timepoints
#'
#' Present value of a series of payoffs for a single given cohort, entering at given future time, allowing for dynamic pricing. This function is a wrapper for [dynpv()] restricted to evaluation of a single cohort.
#' @seealso [dynpv()]
#' @inherit dynpv params return
#' @export
#' @examples
#' library(dplyr)
#'
#' # Obtain dataset
#' democe <- get_dynfields(
#'    heemodel = oncpsm,
#'    payoffs = c("cost_daq_new", "cost_total", "qaly"),
#'    discount = "disc"
#'    )
#'
#' # Obtain discount rate
#' discrate <- get_param_value(oncpsm, "disc")
#'
#' # Obtain payoff vector of interest
#' payoffs <- democe |>
#'    filter(int=="new") |>
#'    mutate(cost_oth_rup = cost_total_rup - cost_daq_new_rup)
#' Nt <- nrow(payoffs)
#'
#' # Run calculation for times 0-9
#' fpv <- futurepv(
#'   tzero = (0:9)*52,
#'   payoffs = payoffs$cost_oth_rup,
#'   prices = 1.001^(1:(2*Nt)-1), # Approx 5.3% every 52 steps
#'   discrate = 0.001 + discrate
#' )
#' fpv
#' summary(fpv)
futurepv <- function(tzero=0, payoffs, prices, discrate){
  # Wrapper for dynpv with uptakes=1 and horizon=length(payoffs)
  dynpv(
    uptakes = 1,
    payoffs = payoffs,
    horizon = length(payoffs),
    tzero = tzero,
    prices = prices,
    discrate = discrate
  )
}

#' @importFrom dplyr across all_of as_tibble bind_rows filter join_by left_join mutate rename select summarize
#' @importFrom tidyr expand_grid pivot_wider
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

#  Copyright (c) 2025 Merck & Co., Inc., Rahway, NJ, USA and its affiliates.
#  All rights reserved.
#
#  This file is part of the dynamicpv program.
#
#  dynamicpv is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Classes and methods for dynpv
# =============================

#' Method to add two dynpv objects together
#'
#' Add together two objects each of class "dynpv"
#'
#' @inherit addprod params details return
#'
#' @export
"+.dynpv" <- function(e1, e2) addprod(e1, e2, mult=1)

#' Method to subtract one dynpv object from another
#'
#' Subtract one object of S7 class "dynpv" from another
#'
#' Present value of `e1-e2` is the present values from `e1` less that from `e2`.
#' Total uptake of `e1-e2` is the uptake from `e1` less that from `e2`. Take
#' care of this when using `$mean` of the summed object.
#'
#' @inherit addprod params return
#' @export
"-.dynpv" <- function(e1, e2) addprod(e1, e2, mult=-1)

#' Method to add two dynpv objects together
#'
#' Add together two objects each of S3 class "dynpv": e1 + mult * e2
#'
#' @param e1 First "dynpv" object
#' @param e2 Second "dynpv" object
#' @param mult Numeric
#'
#' Present value is present value from `e1` plus `mult` times the present value
#' from `e2`. Total uptake is the uptake from `e1` plus `mult` times the uptake
#' from `e2`. Take care of this when using `$mean` of the summed object.
#'
#' @seealso [dynpv()], [futurepv()]
#' @returns S3 object of class "dynpv"
#'
#' @export
addprod <- function(e1, e2, mult) {
  # Avoid no visible binding note
  j <- k <- l <- dpvno <- uj <- pv <- uj_x <- uj_y <- pv_x <- pv_y <- NULL
  # Pull out xdata and subset of ydata; add dpvno=1 or 2 depending on source
  xdata <- e1 |> mutate(dpvno="x")
  ydata <- e2 |> mutate(dpvno="y")
  # Check that j, k and l vectors align
  if (length(xdata$j) != length(ydata$j)) {warning("Uptake vectors differ in length")}
  stopifnot("Length of payoffs vectors differ" = max(xdata$k) == max(ydata$k))
  stopifnot("Length of time-offset vectors (tzero) differ" = max(xdata$l) == max(ydata$l))
  # Combine data
  jdata <- bind_rows(xdata, ydata) |>
    # Spread
    pivot_wider(
      id_cols = c(j, k, l, t),
      names_from = dpvno,
      values_from = c(uj, pv),
      values_fill = 0
    ) |>
    # Sum the present values from both objects
    mutate(
      uj = uj_x + mult * uj_y,
      pv = pv_x + mult * pv_y
    ) |>
    select(-uj_x, -uj_y, -pv_x, -pv_y) |>
    as_tibble()
  class(jdata) <- c("dynpv", class(jdata))
  return(jdata)
}

#' Number of cohorts of uptaking patients
#' 
#' Number of cohorts of uptaking patients, calculated as the length of the `uptakes` input to [dynpv()]
#'  
#' @param df Tibble of class "dynpv" created by [dynpv()] or [futurepv()]
#' @seealso [dynpv()], [futurepv()]
#' @return A number
#'
#' @export
ncoh <- function(df) max(df$k)

#' Number of times at which present value calculations are performed
#'
#' Number of times at which present value calculations are performed, calculated as the length of the `tzero` input to [dynpv()]
#' 
#' @inherit ncoh params return
#' @seealso [dynpv()], [futurepv()]
#' @export
ntimes <- function(df) length(unique(df$l))

#' Total number of uptaking patients
#'
#' Total number of uptaking patients, calculated as the sum of the `uptake` input to [dynpv()], or \eqn{\sum_{j=1}^T u_j}
#' 
#' @inherit ncoh params
#' @seealso [dynpv()], [futurepv()]
#' @return A number or tibble
#'
#' @export
uptake <- function(df) {
  # Avoid no visible binding note
  uj <- sd <- j <- NULL
  tempout1 <- df |>
    summarize(mean=mean(uj), sd=sd(uj), .by=c(j)) |>
    summarize(uptake=sum(mean))
  return(tempout1$uptake)
}

#' Present value for each uptake cohort and calculation time
#'
#' Calculates the sum of the Present Value by uptake cohort (`j`) and time at which the calculation is performed (`tzero` input to [dynpv()])
#' 
#' The [Present Value](https://en.wikipedia.org/wiki/Present_value) of a cashflow \eqn{p_k} for the \eqn{u_j} patients who began treatment at time \eqn{j} and who are in their \eqn{k}th timestep of treatment is as follows
#' \deqn{PV(j,k,l) = u_j \cdot p_k \cdot R_{j+k+l-1} \cdot (1+i)^{2-j-k}}
#' where \eqn{i} is the risk-free discount rate per timestep, \eqn{p_k} is the cashflow amount in today’s money, and \eqn{p_k \cdot R_{j+k+l-1}} is the nominal amount of the cashflow at the time it is incurred, allowing for an offset of \eqn{l = tzero}.
#' 
#' This method returns \eqn{\sum_{k=1}^{T-j+1} PV(j,k,l)} for each value of \eqn{j} and \eqn{l}, where \eqn{T} is the time horizon of the calculation.
#' 
#' @inherit uptake params return
#' @seealso [dynpv()], [futurepv()]
#' @return A number or tibble
#' @export
sum_by_coh <- function(df) {
  # Avoid no visible binding note
  pv <- j <- l <- NULL
  tempout2 <- df |>
    # Summing over k, where uj does not vary by k
    summarize(spv = sum(pv), .by=c(j, l)) |>
    rename(tzero = l)
  result <- if (nrow(tempout2)==1) tempout2$spv else tempout2
  return(result)
}

#' Total present value
#' 
#' Sum of the Present Value, by time at which the calculation is performed (`tzero` input to [dynpv()])
#' 
#' The [Present Value](https://en.wikipedia.org/wiki/Present_value) of a cashflow \eqn{p_k} for the \eqn{u_j} patients who began treatment at time \eqn{j} and who are in their \eqn{k}th timestep of treatment is as follows
#' \deqn{PV(j,k,l) = u_j \cdot p_k \cdot R_{j+k+l-1} \cdot (1+i)^{2-j-k}}
#' where \eqn{i} is the risk-free discount rate per timestep, \eqn{p_k} is the cashflow amount in today’s money, and \eqn{p_k \cdot R_{j+k+l-1}} is the nominal amount of the cashflow at the time it is incurred, allowing for an offset of \eqn{l = tzero}.
#' 
#' The total present value by time at which the calculation is performed, \eqn{TPV(l)}, is therefore the sum of \eqn{PV(j,k,l)} over all \eqn{j} and \eqn{k} within the time horizon \eqn{T}, namely:
#' \deqn{TPV(l) = \sum_{j=1}^{T} \sum_{k=1}^{T-j+1} PV(j,k, l) \\
#' \;
#' = \sum_{j=1}^{T} \sum_{k=1}^{T-j+1} u_j \cdot p_k \cdot R_{l+j+k-1} \cdot (1+i)^{2-j-k}}
#'
#' @inherit uptake params return
#' @seealso [dynpv()], [futurepv()]
#' @return A number or tibble
#' @export
total <- function(df) {
  # Avoid no visible binding note
  pv <- l <- NULL
  tempout3 <- df |>
    summarize(total = sum(pv), .by=c(l)) |>
    rename(tzero = l)
  result <- if (nrow(tempout3)==1) tempout3$total else tempout3
  return(result)
}

#' Mean present value per uptaking patient
#' 
#' Mean of the Present Value per uptaking patient, by time at which the calculation is performed (`tzero` input to [dynpv()]).
#' 
#' This is equal to [total()] divided by [uptake()].
#'
#' @param x Tibble of class "dynpv" created by [dynpv()] or [futurepv()]
#' @param ... Currently unused
#'
#' @inherit uptake return
#' @seealso [dynpv()], [futurepv()]
#' @export
mean.dynpv <- function(x, ...) {
  # Avoid no visible binding note
  tzero <- NULL
  total <- total(x)
  uptake <- uptake(x)
  if (length(total)==1) {
    total / uptake
  } else {
    total |>
      mutate(
        uptake = uptake,
        mean = total / uptake
      ) |>
    select(-total, -uptake)
  }
}

#' Summarize a dynpv object
#'
#' @param object Tibble of class "dynpv" created by [dynpv()] or [futurepv()]
#' @param ... Currently unused
#'
#' @return A list of class "dynpv_summary" with the following elements:
#' - `ncoh`: Number of cohorts of uptaking patients, from [ncoh()]
#' - `ntimes`: Number of times at which present value calculations are performed, from [ntimes()]
#' - `uptake`: Total number of uptaking patients, from [uptake()]
#' - `sum_by_coh`: Present value for each uptake cohort and calculation time, from [sum_by_coh()]
#' - `total`: Total present value, from [total()]
#' - `mean`: Mean present value per uptaking patient, from [mean()], equal to `total`/`uptake`.
#' @seealso [dynpv()], [futurepv()]
#' @export
summary.dynpv <- function(object, ...) {
  structure(
    class = "dynpv_summary",
    list(
      ncoh = ncoh(object),
      ntimes = ntimes(object),
      uptake = uptake(object),
      sum_by_coh = sum_by_coh(object),
      total = total(object),
      mean = mean(object)
    )
  )
}

#' @export
print.dynpv_summary <- function(x, ...) {
  cat("Summary of Dynamic Pricing and Uptake\n")
  cat("     Number of cohorts:            ", x$ncoh, "\n")
  cat("     Number of times:              ", x$ntimes, "\n")
  cat("     Total uptake:                 ", x$uptake, "\n")
  # Output depends on whether $ntimes>1
  if (x$ntimes>1) {
    # Avoid no visible binding note
    tzero <- NULL
    # Create a tibble
    tib <- x$total |>
      left_join(x$mean, join_by(tzero)) |>
      select(tzero, total, mean)
    cat("\n Total and mean present values by timepoint: \n")
    print(tib)
  }
  else {
    cat("     Total present value:          ", x$total, "\n")
    cat("     Mean present value:           ", x$mean, "\n")
  }
  return(invisible(x))
}

#  Copyright (c) 2025 Merck & Co., Inc., Rahway, NJ, USA and its affiliates.
#  All rights reserved.
#
#  This file is part of the dynamicpv program.
#
#  dynamicpv is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  =====================================================================

#' Heemod cost-effectiveness model example
#' 
#' An example three state cost-effectiveness model in oncology built using [heemod::heemod-package()] according to the assumptions and specification in the accompanying paper.
#' 
#' @format ## `oncpsm`
#' A heemod object
#' @source Created based on assumptions.
"oncpsm"

# Use the purrr package
library(purrr)

# Create a quiet function
qget_dynfields <- purrr::quietly(get_dynfields)
qread_csv <- purrr::quietly(readr::read_csv)

# Run the function and expect no error
test_that("Run function and expect no error", {
expect_no_error(
  qget_dynfields(
    heemodel = oncpsm,
    payoffs = c("cost_daq_new", "cost_total", "qaly"),
    discount = "disc"
    )
  )
})

# Check that the exported CSV file matches the original object
test_that("Exported CSV matches original object", {
  # Expect
  testexpect <- qget_dynfields(
    heemodel = oncpsm,
    payoffs = c("cost_daq_new", "cost_total", "qaly"),
    discount = "disc",
    fname = "dyntest"
    )
  # Actual
  testactual <- qread_csv("dyntest.csv")
  # Test
  expect_equal(testexpect$result, testactual$result)
  # Remove the created file
  file.remove("dyntest.csv")
})

---
title: "Mathematical Framework"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{math-framework}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: bibliography.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction and Motivation

Single cohort cost-effectiveness models are routinely used in the decision making of Health Technology Assessment (HTA) bodies, and are widely published in the scientific literature.[@espinosa_2024; @enright_2025; @do_2021] Despite their utility, such models have been criticised as overly limited in scope, omitting important elements of value[@breslau_2023; @shafrin_2024]] and health equity[@avancena_2021]. Anticipated pricing dynamics are routinely ignored, meaning that the long run opportunity cost for drugs may be misrepresented,[@neumann_2022] and single cohort modeling is criticized as not tailored to properly inform decision-making that will impact future cohorts of patients[@hoyle_anderson_2010]. Case studies have shown how modeling pricing and uptake as dynamic can have substantial effects on reported Incremental Cost-Effectiveness Ratio (ICER) values. Without accounting for these effects, ICERs are overstated and unrepresentative.[@schottler_2023; @whittington_2025a; @moreno_2016]

In a recent review, Puls et al listed four challenges in modeling life cycle drug pricing and offered some proposals.[@puls_2024] Pricing changes before Loss of Exclusivity (LoE) events are ‘usually small’ but local pricing data can be informative, whereas after LoE, changes to pricing ‘should be informed by country-specific and historical estimates of average price reductions’, such as may be found in recent reviews.[@lin_2025; @bonet_2025; @serra_2024; @laube_2024] Following earlier recommendations by by Hoyle and Anderson, cost-effectiveness evaluations should include future incident cohorts in addition to the present, prevalent cohort,[@hoyle_anderson_2010] though assumptions may need to be simplified to facilitate calculation. Reporting should include individual and multiple cohorts, assuming uniform or utilization-informed weightings.[@puls_2024]

There are now a growing number of publications of cost-effectiveness evaluations with dynamic pricing and uptake.[@puls_2024; @schottler_2023; @whittington_2025a; @shafrin_2024; @moreno_2016] The purpose of this R package is to provide a simple tool to conduct calculations of present values that allow for dynamic pricing and dynamic uptake. The mathematical framework presented in this vignette formalizes what others have developed and applied,[@hoyle_anderson_2010;@shafrin_2024] and provides the technical basis of the calculations within the `dynamicpv` package. Other vignettes cover calculations of Net Present Values, and illustrate how cost-effectiveness and budget impact models can account for dynamic pricing and uptake. The present scope of the package is models in discrete time only.

## What are dynamic pricing and dynamic uptake?

**Dynamic pricing** is pricing of a resource (e.g. acquisition cost of a medicine, wages of a professional) that changes in time. The opposite of dynamic pricing is static pricing. We may assume prices are static in nominal terms - they are constant and do not change value in time at all; or we may assume prices are static in real terms, in which case, were it not for price inflation, prices would be constant. Alternatively, pricing can be expected to be quite irregular - such as when drug prices dramatically reduce after branded products lose exclusivity. Reflecting these dynamics complicates calculations of Net Present Value, but is more realistic.

**Dynamic uptake** refers to the modeling of multiple series of payoffs/cashflows over time, rather than the modeling of just one series of payoffs/cashflows at a time. This may arise, for example, in considering the treatment costs of multiple cohorts of patients beginning their treatment courses at different times, leading to the term 'stacked cohorts'. There are also analogies to 'run-off triangles' used by insurance actuaries to model the emergence of insurance claims over time following claim events. This is already commonly a consideration in budget impact models in healthcare, but also relevant to cost-effectiveness.[@sullivan_2014]

## Framework

The framework for these calculations is centered around the idea of costs and outcomes accruing to patients as they receive treatment, or more generally experience the consequences of an intervention. Time may be partitioned therefore between the time to begin treatment and the time since starting treatment.

More formally, suppose $j=1, ..., J$ indexes the time at which the patient begins treatment (e.g. a new intervention or Standard of Care, SoC). Suppose $k=1, ..., K$ indexes time since initiating treatment. Time is $t=j+k-1$, and we are interested in $t=1,...,T$, where $T$ is the time horizon of the decision-maker.

This can be illustrated through an example. Suppose then we are considering a cashflow in timestep $t=3$. This will comprise:

- patients who are in the third timestep of treatment that began in timestep 1, $(j,k) = (1, 3)$;
- patients who are in the second timestep of treatment that began in timestep 2, $(j,k) = (2, 2)$; and
- patients who are in the first timestep of treatment that began in timestep 3, $(j,k) = (3, 1)$.

The Present Value of a cashflow $p_k$ for the $u_j$ patients who began treatment at time $j$ and who are in their $k$th timestep of treatment is as follows

$$ PV(j,k) = u_j \cdot p_k \cdot R_{j+k-1} \cdot (1+i)^{2-j-k} $$

where $i$ is the risk-free discount rate per timestep, and $p_k$ is the cashflow amount in today's money, and $p_k \cdot R_{j+k-1}$ is the nominal amount of the cashflow at the time it is incurred.

The total present value is therefore the sum over all $j$ and $k$ within the time horizon $T$. The full function used by the package allows for additional offsetting of the price uprating factor $R$ by time $l$. This can be useful for calculating present values at different present times.

$$ TPV(l) = \sum_{j=1}^{T} \sum_{k=1}^{T-j+1} PV(j, k, l) = \sum_{j=1}^{T} \sum_{k=1}^{T-j+1} u_j \cdot p_k \cdot R_{j+k+l-1} \cdot (1+i)^{2-j-k} $$

## Overview of package

The `dynamicpv::dynpv()` function operationalizes the formula above. It produces output of the class 'dynpv', to which the following methods may be applied.

| Method | Description | Value |
|--------|-------------|-------|
| `mean()` | Mean present value per uptaking patient | $TPV(l) / \sum_{j=1}^{T} u_j =$ `total()`/`uptake()` |
| `ncoh()` | Number of cohorts of uptaking patients | $n({u_j})$ |
| `ntimes()` | Number of times at which present value calculations are performed | $n({l})$ |
| `sum_by_coh()` | Present value for each uptake cohort `j` and calculation time `l` | $\sum_{k=1}^{T-j+1} PV(j,k,l)$ |
| `summary()` | Summarize a `dynpv` object | Text |
| `total()` | Total present value | $TPV(l)$ |
| `uptake()` | Total number of uptaking patients | $\sum_{j=1}^{T} u_j$ |

Also, methods `+` and `-` can be used to add and subtract two `dynpv` objects. It is important to note when interpreting the `mean()` that the arithmetic operations add/subtract the `total()` and `uptake()` values of the objects.

The `dynamicpv::futurepv()` function calculates the present value of a series of payoffs for a single given cohort, entering at given future time, allowing for dynamic pricing. This function is a wrapper for `dynpv()` restricted to evaluation of a single cohort.

## References

---
title: "Cost-Effectiveness Applications"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Cost-Effectiveness Applications}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: bibliography.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

# Numeric notation in this document
# With thanks to https://stackoverflow.com/questions/18965637/set-global-thousand-separator-on-knitr/18967590#18967590
knitr::knit_hooks$set(
    inline = function(x) {
        if(!is.numeric(x)){
            x
        } else if(x<100) {
            prettyNum(round(x, 3), big.mark=",")
        } else {
            prettyNum(round(x, 0), big.mark=",")
        }
    }
)
```

## Introduction

This vignette outlines how to evaluate a cost-effectiveness model in a manner that allows for dynamic pricing and dynamic uptake. Let us consider in turn:

- the decision problem we are modeling through a cost-effectiveness analysis
- dynamic patient uptake
- dynamic pricing

## Methods and Assumptions

We wish to evaluate the cost-effectiveness, measured as incremental cost per QALY, of a new intervention compared to the standard of care (SoC). The model is a partitioned survival analysis typical in oncology with three health states: progression-free (PF), progressive disease (PD) and death, with additional assumptions as follows.

### General assumptions

- The time horizon is 20 years, a timestep is 1 week, discounting is at 3% per year, with an effective date of calculation is 2025-06-01.
- For SoC, progression-free survival (PFS) is modeled as an exponential distribution with mean of 50 weeks; overall survival (OS) in weeks is modelled according to a lognormal distribution with mean and standard deviation on the log scale as 4 and 1 respectively.
- The effect of the new intervention relative to the SoC is a hazard ratio of 0.5 on PFS and 0.6 on OS.
- Health utility is 0.8 in the PF state and 0.6 in the PD state.
- Drug acquisition costs are $400 and $1500 per week for SoC and the new intervention respectively, incurred throughout a patient’s time in the PF state (treat to progression).
- Drug administration costs are $50 and $75 per week on treatment for SoC and the new intervention respectively.
- Adverse events occur in 0.08% and 0.10% of SoC and new intervention patients per week respectively, while on treatment, at an average cost of $2000 per event.
- Disease management costs are $80 per week in PF and $20 per week in PD.
- Patients receive the treatment of interest (new or SoC) until progression. Subsequent treatment costs are $1200 and $300 per week per patient while in the PD state for SoC and new intervention patients respectively.

### Dynamic pricing

Let us suppose the following assumptions concerning pricing:

- Costs are assumed to increases in line with general inflation (2.5% per year), except for effects on drug acquisition costs due to LoEs.
- The date of calculation is 2025-09-01.
- The LoE for the SoC is assumed to occur first, at 2028-01-01, after which there is anticipated to be a 70% reduction in prices over one year.
- The new intervention has an LoE occuring three years later, at 2031-01-01, after which there would be a 50% reduction in prices over one year.

### Dynamic uptake

Let us suppose the following assumptions concerning patient uptake. The aim here is to estimate the incidence of patients for whom the decision problem applies, i.e. the patients who would receive the new intervention, were it made available. This will be a function of, but not the same as, disease incidence or prevalence. We assume:

- Only newly incident patients with the cancer being modeled would be eligible for the new treatment. Existing/prevalent patients with the condition would not be eligible.
- The disease incidence is 1 patient per week.
- Among these patients, were the new intervention to be made available, uptake of the new intervention would be expected to rise linearly from 0% to 100% after 2 years.

In this way, uptake would be gradually increasing with time, accounting for disease epidemiology and the share of patients who receive the new intervention.

## Implementation

### Set-up

First we load the packages necessary for this vignette.

```{r setup, echo=TRUE, message=FALSE}
library(dplyr)
library(ggplot2)
library(lubridate)
library(flexsurv)
library(heemod)
library(tidyr)
library(dynamicpv)
```

We code the time constants, time horizon, discount rates and inflation rates first.

```{r constants}
# Time constants
days_in_year <- 365.25
days_in_week <- 7
cycle_years <- days_in_week / days_in_year # Duration of a one week cycle in years

# Time horizon (years) and number of cycles
thoz <- 20
Ncycles <- ceiling(thoz/cycle_years)

# Real discounting
disc_year <- 0.03 # Per year
disc_cycle <- (1+disc_year)^(cycle_years) - 1 # Per cycle

# Price inflation
infl_year <- 0.025 # Per year
infl_cycle <- (1+infl_year)^(cycle_years) - 1 # Per cycle

# Nominal discounting
nomdisc_year <- (1+disc_year)*(1+infl_year) - 1
nomdisc_cycle <- (1+nomdisc_year)^(cycle_years) - 1 # Per cycle
```

### Cost-effectiveness model

The cost-effectiveness model may then be coded in [heemod](https://cran.r-project.org/package=heemod) as follows.

```{r heemod}
# State names
state_names = c(
  progression_free = "PF",
  progression = "PD",
  death = "Death"
  )

# PFS distribution for SoC with Exp() distribution and mean of 50 weeks
surv_pfs_soc <- heemod::define_surv_dist(
  distribution = "exp",
  rate = 1/50
)

# OS distribution for SoC with Lognorm() distribution, meanlog = 4.5, sdlog = 1
# This implies a mean of exp(4 + 0.5 * 1^2) = exp(4.5) = 90 weeks
surv_os_soc <- heemod::define_surv_dist(
  distribution = "lnorm",
  meanlog = 4,
  sdlog = 1
)

# PFS and OS distributions for new
surv_pfs_new <- heemod::apply_hr(surv_pfs_soc, hr=0.5)
surv_os_new <- heemod::apply_hr(surv_os_soc, hr=0.6)

# Define partitioned survival model, soc
psm_soc <- heemod::define_part_surv(
  pfs = surv_pfs_soc,
  os = surv_os_soc,
  terminal_state = FALSE,
  state_names = state_names
  )

# Define partitioned survival model, soc
psm_new <- heemod::define_part_surv(
  pfs = surv_pfs_new,
  os = surv_os_new,
  terminal_state = FALSE,
  state_names = state_names
  )

# Parameters
params <- heemod::define_parameters(
  # Discount rate
  disc = disc_cycle,
  # Disease management costs
  cman_pf = 80,
  cman_pd = 20,
  # Drug acquisition costs - the SoC regime only uses SoC drug, the New regime only uses New drug
  cdaq_soc = dispatch_strategy(
    soc = 400,
    new = 0
  ),
  cdaq_new = dispatch_strategy(
    soc = 0,
    new = 1500
  ),
  # Drug administration costs
  cadmin = dispatch_strategy(
    soc = 50,
    new = 75
  ),
  # Adverse event risks
  risk_ae = dispatch_strategy(
    soc = 0.08,
    new = 0.1
  ),
  # Adverse event average costs
  uc_ae = 2000,
  # Subsequent treatments
  csubs = dispatch_strategy(
    soc = 1200,
    new = 300
  ),
  # Health state utilities
  u_pf = 0.8,
  u_pd = 0.6,
)

# Define PF states
state_PF <- heemod::define_state(
  # Costs for the state
  cost_daq_soc = discount(cdaq_soc, disc_cycle),
  cost_daq_new = discount(cdaq_new, disc_cycle),
  cost_dadmin = discount(cadmin, disc_cycle),
  cost_dman = discount(cman_pf, disc_cycle),  
  cost_ae = risk_ae * uc_ae,
  cost_subs = 0,
  cost_total = cost_daq_soc + cost_daq_new + cost_dadmin + cost_dman + cost_ae + cost_subs,
  # Health utility, QALYs and life years
  pf_year = discount(cycle_years, disc_cycle),
  life_year = discount(cycle_years, disc_cycle),
  qaly = discount(cycle_years * u_pf, disc_cycle)
  )

# Define PD states
state_PD <- heemod::define_state(
  # Costs for the state
  cost_daq_soc = 0,
  cost_daq_new = 0,
  cost_dadmin = 0,
  cost_dman = discount(cman_pd, disc_cycle),  
  cost_ae = 0,
  cost_subs = discount(csubs, disc_cycle),
  cost_total = cost_daq_soc + cost_daq_new + cost_dadmin + cost_dman + cost_ae + cost_subs,
  # Health utility, QALYs and life years
  pf_year = 0,
  life_year = heemod::discount(cycle_years, disc_cycle),
  qaly = heemod::discount(cycle_years * u_pd, disc_cycle)
  )

# Define Death state
state_Death <- heemod::define_state(
  # Costs are zero
  cost_daq_soc = 0,
  cost_daq_new = 0,
  cost_dadmin = 0,
  cost_dman = 0,
  cost_ae = 0,
  cost_subs = 0,
  cost_total = cost_daq_soc + cost_daq_new + cost_dadmin + cost_dman + cost_ae + cost_subs,
  # Health outcomes are zero
  pf_year = 0,
  life_year = 0,
  qaly = 0,
)

# Define strategy for SoC
strat_soc <- heemod::define_strategy(
    transition = psm_soc,
    "PF" = state_PF,
    "PD" = state_PD,
    "Death" = state_Death
  )

# Define strategy for new
strat_new <- heemod::define_strategy(
  transition = psm_new,
  "PF" = state_PF,
  "PD" = state_PD,
  "Death" = state_Death
)

# Create heemod model
heemodel <- heemod::run_model(
  soc = strat_soc,
  new = strat_new,
  parameters = params,
  cycles = Ncycles,
  cost = cost_total,
  effect = qaly,
  init = c(1, 0, 0),
  method = "life-table"
)
```

### Dynamic pricing

The dynamic pricing assumptions can be codified as follows.

```{r dynpricing}
# Dates
# Date of calculation = 1 September 2025
doc <- lubridate::ymd("20250901")
# Date of LOE for SoC = 1 January 2028
loe_soc_start <- lubridate::ymd("20280101")
# Maturation of SoC prices by LOE + 1 year, i.e. = 1 January 2029
loe_soc_end <- lubridate::ymd("20290101") 
# Date of LOE for new treatment = 1 January 2031
loe_new_start <- lubridate::ymd("20310101")
# Maturation of new treatment prices by LOE + 1 year, i.e. = 1 January 2032
loe_new_end <- lubridate::ymd("20320101") 

# Effect of LoEs on prices once mature
loe_effect_soc <- 0.7
loe_effect_new <- 0.5

# Calculation of weeks since DoC for LoEs and price maturities
wk_start_soc <- floor((loe_soc_start-doc) / lubridate::dweeks(1))
wk_end_soc <- floor((loe_soc_end-doc) / lubridate::dweeks(1))
wk_start_new <- floor((loe_new_start-doc) / lubridate::dweeks(1))
wk_end_new <- floor((loe_new_end-doc) / lubridate::dweeks(1))

# Price maturity times
wk_mature_soc <- wk_end_soc - wk_start_soc
wk_mature_new <- wk_end_new - wk_start_new

# Create a tibble of price indices of length 2T, then pull out columns as needed
# We only need of length T for now, but need of length 2T for future calculations later
pricetib <- tibble(
  model_time = 1:(2*Ncycles),
  model_year = model_time * cycle_years,
  static = 1,
  geninfl = (1 + infl_cycle)^(model_time - 1),
  loef_soc = pmin(pmax(model_time - wk_start_soc, 0), wk_mature_soc) / wk_mature_soc,
  loef_new = pmin(pmax(model_time - wk_start_new, 0), wk_mature_new) / wk_mature_new,
  dyn_soc = geninfl * (1 - loe_effect_soc * loef_soc),
  dyn_new = geninfl * (1 - loe_effect_new * loef_new)
)

# Price indices required for calculations
prices_oth <- pricetib$geninfl
prices_static <- pricetib$static
prices_dyn_soc <- pricetib$dyn_soc
prices_dyn_new <- pricetib$dyn_new
```

### Dynamic uptake

 The dynamic uptake assumptions can be codified as follows.

```{r dynuptake}
# Time for uptake to occur
uptake_years <- 2

# Uptake vector for non-dynamic uptake
uptake_single <- c(1, rep(0, Ncycles-1))

# Uptake vector for dynamic uptake
uptake_weeks <- round(uptake_years / cycle_years)
share_multi <- c((1:uptake_weeks)/uptake_weeks, rep(1, Ncycles-uptake_weeks))
uptake_multi <- rep(1, Ncycles) * share_multi
```

## Results

### Without dynamic pricing or dynamic uptake

The conventional cost-effectiveness model is static.

```{r static1}
heemodel
```

Let's examine each payoff more closely. The next steps extract a payoff vector from the model object. The model/object contains several payoffs accumulated in each timestep, calculated as at time zero: 

- drug acquisition cost of the standard of care (*cost_daq_soc*),
- drug acquisition cost of the new intervention (*cost_daq_new*),
- cost of drug administration (*cost_dadmin*),
- cost of disease management (*cost_dman*),
- cost of treating adverse events (*cost_ae*),
- cost of subsequent lines of treatment (*cost_subs*),
- total cost (*cost_total*),
- progression-free life years (*pf_year*),
- life years (*life_year*), and
- QALYs (*qaly*).

The `dynamicpv::get_dynfields()` function extracts these parameters from the [heemod](https://cran.r-project.org/package=heemod) model object, and calculates 'rolled-up' values as at the start of each timestep rather than discounted to time zero. The rolled-up values are what `dynamicpv::dynpv()` requires.

```{r static2}
# Pull out the payoffs of interest from oncpsm
payoffs <- get_dynfields(
    heemodel = heemodel,
    payoffs = c("cost_daq_new", "cost_daq_soc", "cost_total", "qaly", "life_year"),
    discount = "disc"
    ) |>
    mutate(
      model_years = model_time * cycle_years,
      # Derive costs other than drug acquisition, as at time zero
      cost_nondaq = cost_total - cost_daq_new - cost_daq_soc,
      # ... and at the start of each timestep
      cost_nondaq_rup = cost_total_rup - cost_daq_new_rup - cost_daq_soc_rup
    )

# Create and view dataset for SoC
hemout_soc <- payoffs |> filter(int=="soc")
head(hemout_soc)

# Create and view dataset for new intervention
hemout_new <- payoffs |> filter(int=="new")
head(hemout_new)
```

### Scenario 1: No dynamic uptake or pricing

With non-dynamic uptake, we use `uptakes`=`uptake_single`=1. Drug acquisition costs are constant in real terms (`prices=prices_static`) and are discounted at the risk-free real rate (`discrate=disc_cycle`). Other costs rise in line with general price inflation (`prices=prices_oth`) and are discounted at nominal discount rates (`discrate=nomdisc_cycle`). QALYs are not affected by price inflation (`prices=prices_static`) and are discounted at the risk-free real rate (`discrate=disc_cycle`).

```{r scen1}
# SOC, costs other than drug acquisition
s1_soc_othcost <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_soc$cost_nondaq_rup,
    prices = prices_oth,
    discrate = nomdisc_cycle
    )

# SOC, drug acquisition costs
s1_soc_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_soc$cost_daq_soc_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# SOC, total costs
s1_soc_cost <- s1_soc_daqcost + s1_soc_othcost

# SOC, QALYs
s1_soc_qaly <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_soc$qaly_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# New intervention, costs other than drug acquisition
s1_new_othcost <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_new$cost_nondaq_rup,
    prices = prices_oth,
    discrate = nomdisc_cycle
    )

# New intervention, drug acquisition costs
s1_new_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_new$cost_daq_new_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# New intervention, total costs
s1_new_cost <- s1_new_daqcost + s1_new_othcost

# New intervention, QALYs
s1_new_qaly <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_new$qaly_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# Incremental cost
s1_icost <- s1_new_cost - s1_soc_cost

summary(s1_icost)

# Incremental QALY
s1_iqaly <- s1_new_qaly - s1_soc_qaly

summary(s1_iqaly)

# ICER
s1_icer <- total(s1_icost) / total(s1_iqaly)
s1_icer
```

These results show that the new intervention is associated with $`r total(s1_icost)` incremental costs relative to the standard of care (including $`r total(s1_new_daqcost)` of drug acquisition costs for the new intervention) and `r total(s1_iqaly)` incremental QALYs. The cumulative ICER (incremental cost per QALY) is $`r s1_icer` per QALY at the 20 year time horizon.

### Scenario 2: Dynamic pricing, no dynamic uptake

The costs of drug acquisition in each arm differ in Scenario 2 through applying the relevant dynamic price index (`prices_dyn_soc` and `prices_dyn_new`), with discounting at nominal rates (`discrate = nomdisc_cycle`). Otherwise costs and QALYs are unchanged from Scenario 1.

```{r calc_scen2}
# SOC, costs other than drug acquisition are unchanged
s2_soc_othcost <- s1_soc_othcost

# SoC, drug acquisition costs
s2_soc_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_soc$cost_daq_soc_rup,
    prices = prices_dyn_soc,
    discrate = nomdisc_cycle
    )

# SoC, total costs
s2_soc_cost <- s2_soc_daqcost + s2_soc_othcost

# SoC, QALYs are unchanged
s2_soc_qaly <- s1_soc_qaly

# New intervention, costs other than drug acquisition are unchanged
s2_new_othcost <- s1_new_othcost

# New intervention, drug acquisition costs
s2_new_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_single,
    payoffs = hemout_new$cost_daq_new_rup,
    prices = prices_dyn_new,
    discrate = nomdisc_cycle
    )

# New intervention, total costs
s2_new_cost <- s2_new_daqcost + s2_new_othcost

# New intervention, QALYs are unchanged
s2_new_qaly <- s1_new_qaly

# Incremental cost
s2_icost <- s2_new_cost - s2_soc_cost

summary(s2_icost)

# Incremental QALY
s2_iqaly <- s2_new_qaly - s2_soc_qaly

summary(s2_iqaly)

# ICER
s2_icer <- total(s2_icost) / total(s2_iqaly)
s2_icer
```

Under scenario 2, the new intervention has an incremental cost-effectiveness of $`r s2_icer` per QALY (incremental costs of $`r total(s2_icost)`, incremental QALYs of `r total(s2_iqaly)`).

### Scenario 3: Dynamic uptake, not dynamic pricing

The calculation for Scenario 3 is the same as for Scenario 1 except for dynamic uptake, which is handled by setting `uptakes = uptake_multi`.

```{r calc_scen3}
# SOC, costs other than drug acquisition
s3_soc_othcost <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_soc$cost_nondaq_rup,
    prices = prices_oth,
    discrate = nomdisc_cycle
    )

# SoC, drug acquisition costs
s3_soc_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_soc$cost_daq_soc_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# SoC, total costs
s3_soc_cost <- s3_soc_daqcost + s3_soc_othcost

# SoC, QALYs
s3_soc_qaly <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_soc$qaly_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# New intervention, costs other than drug acquisition
s3_new_othcost <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_new$cost_nondaq_rup,
    prices = prices_oth,
    discrate = nomdisc_cycle
    )

# New intervention, drug acquisition costs
s3_new_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_new$cost_daq_new_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# New intervention, total costs
s3_new_cost <- s3_new_daqcost + s3_new_othcost

# New intervention, QALYs
s3_new_qaly <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_new$qaly_rup,
    prices = prices_static,
    discrate = disc_cycle
    )

# Incremental costs
s3_icost <- s3_new_cost - s3_soc_cost

summary(s3_icost)

# Incremental QALYs
s3_iqaly <- s3_new_qaly - s3_soc_qaly

summary(s3_iqaly)

# ICER
s3_icer <- total(s3_icost) / total(s3_iqaly)
s3_icer
```

Under scenario 3, the new intervention has an incremental cost-effectiveness of $`r s3_icer` per QALY (incremental costs of $`r total(s3_icost)`, incremental QALYs of `r total(s3_iqaly)` from a cohort comprising `r sum(uptake_multi)` patients).

### Scenario 4: Dynamic pricing and uptake

The costs of drug acquisition in each arm differ in Scenario 4 from Scenario 3 through applying the relevant dynamic price index (`prices_dyn_soc` and `prices_dyn_new`). Otherwise costs and QALYs are unchanged from Scenario 3.

```{r calc_scen4}
# SOC, costs other than drug acquisition are unchanged
s4_soc_othcost <- s3_soc_othcost

# SoC, drug acquisition costs
s4_soc_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_soc$cost_daq_soc_rup,
    prices = prices_dyn_soc,
    discrate = nomdisc_cycle
    )

# SoC, total costs
s4_soc_cost <- s4_soc_daqcost + s4_soc_othcost

# SoC, QALYs are unchanged
s4_soc_qaly <- s3_soc_qaly

# New intervention, costs other than drug acquisition are unchanged
s4_new_othcost <- s3_new_othcost

# New intervention, drug acquisition costs
s4_new_daqcost <- dynamicpv::dynpv(
    uptakes = uptake_multi,
    payoffs = hemout_new$cost_daq_new_rup,
    prices = prices_dyn_new,
    discrate = nomdisc_cycle
    )

# New intervention, total costs
s4_new_cost <- s4_new_daqcost + s4_new_othcost

# New intervention, QALYs
s4_new_qaly <- s3_new_qaly

# Incremental costs
s4_icost <- s4_new_cost - s4_soc_cost

summary(s4_icost)

# Incremental QALYs
s4_iqaly <- s4_new_qaly - s4_soc_qaly

summary(s4_iqaly)

# ICER
s4_icer <- total(s4_icost) / total(s4_iqaly)
s4_icer
```

Under scenario 4, the new intervention has an incremental cost-effectiveness of $`r s4_icer` per QALY (incremental costs of $`r total(s4_icost)`, incremental QALYs of `r total(s4_iqaly)` from a cohort comprising `r sum(uptake_multi)` patients).

### Summary of results

Total costs and QALYs for each scenario are summarized in the table below. The results above are skewed by the fact that some scenarios represent more than one patient cohort. Presenting results per patient, allows easier comparison between the scenarios.

|    |  | Scenario 1 | Scenario 2 | Scenario 3 | Scenario 4 |
|:---|:--|------------|------------|------------|------------|
| Dynamic pricing? || No | Yes | No | Yes |
| Dynamic uptake? || No | No | Yes | Yes |
| Cohort size || 1 | 1 | `r sum(uptake_multi)` | `r sum(uptake_multi)` |
| Total costs (cohort) | | ||||
| | New intervention | `r total(s1_new_cost)` | `r total(s2_new_cost)` | `r total(s3_new_cost)` | `r total(s4_new_cost)` |
| | Standard of care | `r total(s1_soc_cost)` | `r total(s2_soc_cost)` | `r total(s3_soc_cost)` | `r total(s4_soc_cost)` |
| | Incremental | `r total(s1_icost)` | `r total(s2_icost)` | `r total(s3_icost)` | `r total(s4_icost)` |
| Total costs (per patient) | | ||||
| | New intervention | `r total(s1_new_cost)` | `r total(s2_new_cost)` | `r total(s3_new_cost)/sum(uptake_multi)` | `r total(s4_new_cost)/sum(uptake_multi)` |
| | Standard of care | `r total(s1_soc_cost)` | `r total(s2_soc_cost)` | `r total(s3_soc_cost)/sum(uptake_multi)` | `r total(s4_soc_cost)/sum(uptake_multi)` |
| | Incremental | `r total(s1_icost)` | `r total(s2_icost)` | `r total(s3_icost)/sum(uptake_multi)` | `r total(s4_icost)/sum(uptake_multi)` |
| Total QALYs (cohort) | ||||
| | New intervention | `r total(s1_new_qaly)` | `r total(s2_new_qaly)` | `r total(s3_new_qaly)` | `r total(s4_new_qaly)` |
| | Standard of care | `r total(s1_soc_qaly)` | `r total(s2_soc_qaly)` | `r total(s3_soc_qaly)` | `r total(s4_soc_qaly)` |
| | Incremental | `r total(s1_iqaly)` | `r total(s2_iqaly)` | `r total(s3_iqaly)` | `r total(s4_iqaly)` |
| Total QALYs (per patient) | ||||
| | New intervention | `r total(s1_new_qaly)` | `r total(s2_new_qaly)` | `r total(s3_new_qaly)/sum(uptake_multi)` | `r total(s4_new_qaly)/sum(uptake_multi)` |
| | Standard of care | `r total(s1_soc_qaly)` | `r total(s2_soc_qaly)` | `r total(s3_soc_qaly)/sum(uptake_multi)` | `r total(s4_soc_qaly)/sum(uptake_multi)` |
| | Incremental | `r total(s1_iqaly)` | `r total(s2_iqaly)` | `r total(s3_iqaly)/sum(uptake_multi)` | `r total(s4_iqaly)/sum(uptake_multi)` |
| ICER | | `r s1_icer` | `r s2_icer` | `r s3_icer` | `r s4_icer` |

: Cost-effectiveness model results by scenario

## Future single cohort ICER

The table above presents the cost-effectiveness results as of the date of calculation, `r doc`. However, it is interesting to explore how the ICER will change over time, given the expected evolution of prices. We use `dynamicpv::futurepv()` to calculate present values at future times. This function is a wrapper for `dynamicpv::dynpv()`. This is the single cohort ICER (no dynamic uptake) but with dynamic pricing, so corresponds with Scenario 2 from earlier.

First, we calculate the costs at each time of interest. Then we calculate the ICER, given the incremental QALYs we have already observed - and which are immune from pricing effects.

```{r future_calc}
# Times at which to plot ICER
gtimes <- round((0:(2*thoz))/cycle_years/2)
# SOC drug acquisition costs
gc_soc_daq <- gtimes |>
    purrr::map_vec(\(l) mean(futurepv(
        tzero = l,
        payoffs = hemout_soc$cost_daq_soc_rup,
        prices = prices_dyn_soc,
        discrate = nomdisc_cycle
    ))
)
# SOC other costs
gc_soc_oth <- gtimes |>
    purrr::map_vec(\(l) mean(futurepv(
        tzero = l,
        payoffs = hemout_soc$cost_nondaq_rup,
        prices = prices_oth,
        discrate = nomdisc_cycle
    ))
)
# New drug acquisition costs
gc_new_daq <- gtimes |>
    purrr::map_vec(\(l) mean(futurepv(
        tzero = l,
        payoffs = hemout_new$cost_daq_new_rup,
        prices = prices_dyn_new,
        discrate = nomdisc_cycle
    ))
)
# New other costs
gc_new_oth <- gtimes |>
    purrr::map_vec(\(l) mean(futurepv(
        tzero = l,
        payoffs = hemout_new$cost_nondaq_rup,
        prices = prices_oth,
        discrate = nomdisc_cycle
    ))
)
# Combine in a tibble
ds <- tibble(
    # Time in weeks and years
    time_weeks = gtimes,
    time_years = time_weeks * cycle_years,
    # Evaluation date
    evaldate = doc + time_weeks * 7,
    # Price/inflation index
    pinfl = prices_oth[gtimes + 1],
    # Total costs for each intervention
    totcost_new = gc_new_daq + gc_new_oth,
    totcost_soc = gc_soc_daq + gc_soc_oth,
    # Scenario 1/2 QALYs
    qaly_soc = total(s1_soc_qaly),
    qaly_new = total(s1_new_qaly)
  ) |>
  mutate(
    # Incremental cost and QALYs
    icost = totcost_new-totcost_soc,
    iqaly = qaly_new-qaly_soc,
    # Nominal ICER, and real (inflation adjusted) ICER
    Nominal = icost/iqaly,
    Real = Nominal / pinfl
  ) |>
  # Pivot to long so can be used in a graphic
  pivot_longer(
    cols = c("Nominal", "Real"),
    names_to = "Type",
    values_to = "ICER"
  )
```

We should check at this point that the ICER we are starting at ($`r ds$ICER[1]`) matches the scenario 2 ICER from earlier ($`r s2_icer`).

The following plot shows the nominal ICER calculated at different times, given the pricing, LoE and other assumptions. The horizontal dotted line confirms that the initial ICER matches the value from Scenario 2. The vertical dashed lines mark the timings of the LoEs of first the standard of care, and then the new treatment.

```{r calc_future}
# Plot real and nominal present value over time
ggplot(ds,
  aes(x = evaldate, y = ICER, color=Type)) +
  geom_line() +
  labs(x = "Evaluation date") +
  geom_hline(yintercept = ds$ICER[1], linetype='dotted') +
  geom_vline(xintercept = loe_new_start, linetype='dashed') +
  geom_vline(xintercept = loe_soc_start, linetype='dashed') +
  scale_y_continuous(
    labels = scales::comma,
    limits=c(0, 150000)
    )
```

## Discussion

The findings for this example model are as follows:

- The effect of assuming dynamic pricing rather than flat pricing over time is shown in the difference between results of scenarios 1 and 2. Scenario 1 assumed no increases in drug acquisition costs, but inflationary increases in other costs. Scenario 2 additionally assumed drug prices of the new intervention and SoC comparator would be eroded due to losses of exclusivity. This has a modest impact on costs per patient of the new intervention (reducing from $`r total(s1_new_cost)` to $`r total(s2_new_cost)`). Accordingly the ICER reduces from $`r s1_icer` to $`r s2_icer` per QALY.

- The effect of modeling uptake dynamically rather than just the current cohort of patients is shown in the difference between results of scenarios 1 and 3. Dynamic modeling of uptake has the effect of weighting the costs and QALYs per patient in scenario 3 towards a later cohort of patients, where the accumulation of costs and QALYs over the time remaining in the payer's 20 year time horizon will be less. The incremental costs per patient reduced from $`r total(s1_new_cost)` in scenario 1 to $`r total(s3_new_cost)/sum(uptake_multi)` in scenario 3, whereas the incremental QALYs reduced from `r total(s1_new_qaly)` to `r total(s3_new_qaly)/sum(uptake_multi)` per patient. Overall, the ICER increased from $`r s1_icer` to $`r s3_icer` per QALY.

- The effects of dynamic pricing and dynamic uptake are extremely synergistic. Scenario 4 illustrates the effects of both dynamic pricing and cohort modeling. The results per patient are identical to scenario 3, except that the cost per patient of the new intervention reduces from $`r total(s3_new_cost)/sum(uptake_multi)` to $`r total(s4_new_cost)/sum(uptake_multi)`, due to the impact of dynamic pricing. The ICER in this case is reduced considerably to $`r s4_icer` per QALY.

- As shown in the Figure, the ICER increases a little at first, as we approach the LoE of the standard of care comparator. After all, a cheaper comparator will increase the incremental cost of the new treatment. However, this is soon dwarfed by the effect of the LoE of the new treatment, which occurs later. This reduces the ICER significantly as the LoE approaches. Thereafter, the ICER is constant in real terms (but increasing with price inflation in nominal terms).

---
title: "Net Present Value"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{net-present-value}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

# Numeric notation in this document
# With thanks to https://stackoverflow.com/questions/18965637/set-global-thousand-separator-on-knitr/18967590#18967590
knitr::knit_hooks$set(
    inline = function(x) {
        if(!is.numeric(x)){
            x
        } else if(x<100) {
            prettyNum(round(x, 3), big.mark=",")
        } else {
            prettyNum(round(x, 0), big.mark=",")
        }
    }
)
```

## Introduction

This vignette describes how to use the `dynamicpv` package to derive (Net) Present Values (NPVs) that account for dynamic (or static) pricing, and dynamic (or static) uptake. We consider:

- NPV with static pricing and static uptake
- NPV with dynamic pricing and static uptake, with prices increasing regularly and irregularly
- NPV with static pricing and dynamic uptake, with constant and variable uptake over time
- NPV with both dynamic pricing and dynamic uptake

First let us load the packages we will use for this vignette.

```{r setup}
library(dynamicpv)
```

## NPV with static pricing and static uptake

The NPV of a cashflow $c_t$ in $t$ years time, assuming prices constant in real terms and an annual discount rate of $i_{real}$ is calculated as follows:

$$NPV = \frac{c_t}{(1+i_{real})^{t}}$$

It is overkill rather to use `dynamicpv` to calculate the NPV of a cashflow that is constant in real terms, since there are simpler routes to doing this. Nevertheless, let us start there, since it helps understand functionality. We use a simple cashflow and $i=3\%$.

```{r assume1}
# A simple cashflow
cashflow <- c(110, 120, 130, 140, 150)

# (Real) discount rate of 3\% per timestep (year)
disc <- 0.03
vt1 <- (1 + disc)^(-1 * (0:4))
```

The calculation can be presented in a table as follows.

| Time | Cashflow | Discount factor    | Product |
|------|----------|--------------------|---------|
| $t$  | $c_t$    | $v^t = (1+i)^{1-t}$ |         |
| 1 | `r cashflow[1]` | `r vt1[1]` | `r cashflow[1] * vt1[1]` |
| 2 | `r cashflow[2]` | `r vt1[2]` | `r cashflow[2] * vt1[2]` |
| 3 | `r cashflow[3]` | `r vt1[3]` | `r cashflow[3] * vt1[3]` |
| 4 | `r cashflow[4]` | `r vt1[4]` | `r cashflow[4] * vt1[4]` |
| 5 | `r cashflow[5]` | `r vt1[5]` | `r cashflow[5] * vt1[5]` |
| Total | `r sum(cashflow)` | | `r sum(vt1 * cashflow)` | 

: Calculation of NPV from a simple cashflow and given discount rate

```{r demo1}
# Run dynpv calculation with full output
pv1 <- dynpv(payoffs=cashflow, discrate=disc)

summary(pv1)
```

The present value of this cashflow is `r total(pv1)`. The same result could have been found more easily than invoking this package.

```{r simpler}
sum(vt1 * cashflow)
```

## NPV with dynamic pricing and static uptake

Cashflows may not in general be expected to be constant in real terms. Accordingly it can be preferable to explicitly estimate the cashflow in nominal terms, $c'_t = c_t \cdot R_t$, and discount with a nominal discount rate that includes expected inflation.

$$ 1+i_{nom} = (1+i_{real})(1+r_g) $$
$$NPV = \frac{c'_t}{(1+i_{nom})^t} = \frac{c_t \cdot R_t}{(1+i_{nom})^t} $$

### Prices that increase regularly

In this example, let us assume that prices of the resource underlying the cashflow increases at 1 \% per year, but that general price inflation is 2.5 \% per year. The real discount rate is unchanged at 3 \% per year, so the nominal discount rate is therefore approximately 5.5 \% per year.

First, we derive a price index to handle the changing price of the resource underlying the cashflow. Then we derive the nominal discount rate, incorporating the general rate of price inflation in the economy. We then combine these inputs with `dynpv()` to calculate the NPV.

```{r calc2a}
# Set up price index
pinfl <- 0.01
pindex <- (1+pinfl)^(0:4)
pindex

# Nominal discount rate
nomdisc <- (1+disc)*1.025-1

# Calculate present value
pv2 <- dynpv(payoffs=cashflow, prices=pindex, discrate=nomdisc)

summary(pv2)
```

The NPV of `r total(pv2)` could still have been calculated using base R functions.

```{r calc2b}
# Compare with more base calculations
vt2 <- (1+nomdisc)^(-1 * (0:4))
sum(vt2 * cashflow * pindex)
```

### Prices that increase irregularly

Where `dynamicpv` is most relevant is when the price index is more irregular, more dynamic. For example, one might anticipate a sudden future change in a price of a certain resource (e.g. drug price reduction on loss of exclusivity). This can be accommodated in the price index. In this simple example, let us assume that the underlying price of the resource reduces to half its original value from the fourth year.

```{r calc3a}
# Revise the price index to be 0.5 from year 4
pindex[4:5] <- 0.5
pindex

# Calculate present value
pv3 <- dynpv(payoffs=cashflow, prices=pindex, discrate=nomdisc)

summary(pv3)
```

With this dynamic pricing change, the NPV has reduced by `r total(pv2) - total(pv3)`, from `r total(pv2)` to `r total(pv3)`. The corresponding base R code is shown below.

```{r calc3b}
# Compare with more base calculations
sum(vt2 * cashflow * pindex)
```

## NPV with static pricing and dynamic uptake

### Uptake is constant over time

Suppose we have one new patient each year, and we wish to calculate the total NPV in a time horizon of 5 years. Now we have a payoff triangle as follows.

| Time | Cashflow 1 | Cashflow 2 | Cashflow 3 | Cashflow 4 | Cashflow 5 | Cashflow Sum | Discount factor | Product |
|------|----------|--------------------|---------|----|-----|----|----|----|
| $t$  | $c_t$    | $c_{t-1}$ | $c_{t-2}$ | $c_{t-3}$ | $c_{t-4}$ |   | $v^t = (1+i)^{1-t}$ |         |
| 1 | `r cashflow[1]` | - | - | - | - | `r cashflow[1]` |`r vt1[1]` | `r cashflow[1] * vt1[1]` |
| 2 | `r cashflow[2]` | `r cashflow[1]` | - | - | - | `r sum(cashflow[1:2])` |`r vt1[2]` | `r sum(cashflow[1:2]) * vt1[2]` |
| 3 | `r cashflow[3]` | `r cashflow[2]` |`r cashflow[1]` | - | - | `r sum(cashflow[1:3])` | `r vt1[3]` | `r sum(cashflow[1:3]) * vt1[3]` |
| 4 | `r cashflow[4]` | `r cashflow[3]` | `r cashflow[2]` |`r cashflow[1]` | - | `r sum(cashflow[1:4])` | `r vt1[4]` | `r sum(cashflow[1:4]) * vt1[4]` |
| 5 | `r cashflow[5]` | `r cashflow[4]` | `r cashflow[3]` | `r cashflow[2]` |`r cashflow[1]` | `r sum(cashflow[1:5])` | `r vt1[5]` | `r sum(cashflow[1:5]) * vt1[5]` |
| Total | `r sum(cashflow)` | | | | | | | `r sum(vt1 * cumsum(cashflow))` | 

: Calculation of NPV from a simple cashflow and given discount rate

With `dynamicpv::dynpv()`, we just update the `uptakes` argument.

```{r uptake2}
# Uptake vector is (1, 1, 1, 1, 1)
uptakes1 <- rep(1, 5)

# NPV calculation
pv4 <- dynpv(payoffs=cashflow, uptakes=uptakes1, discrate=disc)

summary(pv4)
```

There are `r sum(uptakes1)` patients with a total NPV of `r total(pv4)`, which equates to `r mean(pv4)` on average per patient. This can also be calculated in base R.

```{r verify2}
sum(vt1 * cumsum(cashflow))
```

### Uptake varies over time

In general, uptake will not be constant in time. Due to factors such as varying epidemiology (prevalence and incidence), prior treatment pathways (patient testing, patient diagnostics, prior treatment usage), as well as the specific uptake of the treatment of interest (patient share), uptake will also be highly variable over time. In this setting, the total and mean NPVs must reflect weightings applicable to each cashflow.

For example, suppose the number of patients receiving treatment increases by one each year. The weighting given to cashflow 1 would then be $1/(1+2+3+4+5)=6.67$ \%.

```{r uptake3}
# Uptake vector is (1, 2, 3, 4, 5)
uptakes2 <- 1:5

# NPV calculation
pv5 <- dynpv(payoffs=cashflow, uptakes=uptakes2, discrate=disc)

summary(pv5)
```

There are `r sum(uptakes2)` patients with a total NPV of `r total(pv5)`, which equates to `r mean(pv5)` on average per patient. Verifying this result in base R also becomes more complicated.

```{r verify3}
# Verifying total NPV is now more complicated
checkpv <- rep(0, 5)
for (i in 1:5) {
  checkpv[i] <- sum(cashflow[1:i] * uptakes2[i:1] * vt1[i])
}
sum(checkpv)
```

## NPV with dynamic pricing and dynamic uptake

The package becomes most powerful when considering both dynamic uptake and dynamic pricing. A simple call to `dynamicpv::dynpv()` replaces what would be rather more complicated with base functions, even in this toy example.

```{r uptake4}
# NPV calculation
pv6 <- dynpv(payoffs=cashflow, uptakes=uptakes2, prices=pindex, discrate=nomdisc)

# Calculation results
pv6

# Total present value = sum of the pv column
sum(pv6$pv)

summary(pv6)
```

There are `r sum(uptakes2)` patients with a total NPV of `r total(pv6)`, which equates to `r mean(pv6)` on average per patient.

---
title: "Budget Impact Applications"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Budget Impact Applications}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

# Numeric notation in this document
# With thanks to https://stackoverflow.com/questions/18965637/set-global-thousand-separator-on-knitr/18967590#18967590
knitr::knit_hooks$set(
    inline = function(x) {
        if(!is.numeric(x)){
            x
        } else if(x<100) {
            prettyNum(round(x, 3), big.mark=",")
        } else {
            prettyNum(round(x, 0), big.mark=",")
        }
    }
)
```

## Introduction

Our objective is to evaluate the budget impact of introducing a new intervention, assuming static or dynamic pricing. Uptake in budget impact models is already modeled in a dynamic fashion.

## Methods and Assumptions

### General assumptions

Budget impact models conventionally have no discounting and a shorter time horizon than cost-effectiveness models, so we will use a time horizon of 5 years here and a discount rate of 0\%. We will compute a budget impact model using current pricing (per convention) as well as by using dynamic pricing according to the assumptions previously set.

### Dynamic pricing

To recap, we had the following assumptions concerning pricing, with a date of calculation of 2025-09-01.

- Costs are assumed to increases in line with general inflation (2.5% per year), except for effects on drug acquisition costs due to LoEs.
- The LoE for the SoC is assumed to occur first, at 2028-01-01, after which there is anticipated to be a 70% reduction in prices over one year.
- The new intervention has an LoE occuring three years later, at 2031-01-01, after which there would be a 50% reduction in prices over one year.

### Dynamic uptake

We had the following assumptions concerning patient uptake.

- Only newly incident patients with the cancer being modeled would be eligible for the new treatment. Existing/prevalent patients with the condition would not be eligible.
- The disease incidence is 1 patient per week.
- Among these patients, were the new intervention to be made available, uptake of the new intervention would be expected to rise linearly from 0% to 100% after 2 years.

## Implementation

### Set-up

First we load the packages necessary for this vignette.

```{r setup, echo=TRUE, message=FALSE}
library(dplyr)
library(lubridate)
library(heemod)
library(dynamicpv)
```

```{r constants, include=FALSE}
# Time constants
days_in_year <- 365.25
days_in_week <- 7
cycle_years <- days_in_week / days_in_year # Duration of a one week cycle in years

# Time horizon (years) and number of cycles
thoz <- 20
Ncycles <- ceiling(thoz/cycle_years)

# Real discounting
disc_year <- 0.03 # Per year
disc_cycle <- (1+disc_year)^(cycle_years) - 1 # Per cycle

# Price inflation
infl_year <- 0.025 # Per year
infl_cycle <- (1+infl_year)^(cycle_years) - 1 # Per cycle

# Nominal discounting
nomdisc_year <- (1+disc_year)*(1+infl_year) - 1
nomdisc_cycle <- (1+nomdisc_year)^(cycle_years) - 1 # Per cycle
```

```{r heemod, include=FALSE}
# State names
state_names = c(
  progression_free = "PF",
  progression = "PD",
  death = "Death"
  )

# PFS distribution for SoC with Exp() distribution and mean of 50 weeks
surv_pfs_soc <- heemod::define_surv_dist(
  distribution = "exp",
  rate = 1/50
)

# OS distribution for SoC with Lognorm() distribution, meanlog = 4.5, sdlog = 1
# This implies a mean of exp(4 + 0.5 * 1^2) = exp(4.5) = 90 weeks
surv_os_soc <- heemod::define_surv_dist(
  distribution = "lnorm",
  meanlog = 4,
  sdlog = 1
)

# PFS and OS distributions for new
surv_pfs_new <- heemod::apply_hr(surv_pfs_soc, hr=0.5)
surv_os_new <- heemod::apply_hr(surv_os_soc, hr=0.6)

# Define partitioned survival model, soc
psm_soc <- heemod::define_part_surv(
  pfs = surv_pfs_soc,
  os = surv_os_soc,
  terminal_state = FALSE,
  state_names = state_names
  )

# Define partitioned survival model, soc
psm_new <- heemod::define_part_surv(
  pfs = surv_pfs_new,
  os = surv_os_new,
  terminal_state = FALSE,
  state_names = state_names
  )

# Parameters
params <- heemod::define_parameters(
  # Discount rate
  disc = disc_cycle,
  # Disease management costs
  cman_pf = 80,
  cman_pd = 20,
  # Drug acquisition costs - the SoC regime only uses SoC drug, the New regime only uses New drug
  cdaq_soc = dispatch_strategy(
    soc = 400,
    new = 0
  ),
  cdaq_new = dispatch_strategy(
    soc = 0,
    new = 1500
  ),
  # Drug administration costs
  cadmin = dispatch_strategy(
    soc = 50,
    new = 75
  ),
  # Adverse event risks
  risk_ae = dispatch_strategy(
    soc = 0.08,
    new = 0.1
  ),
  # Adverse event average costs
  uc_ae = 2000,
  # Subsequent treatments
  csubs = dispatch_strategy(
    soc = 1200,
    new = 300
  ),
  # Health state utilities
  u_pf = 0.8,
  u_pd = 0.6,
)

# Define PF states
state_PF <- heemod::define_state(
  # Costs for the state
  cost_daq_soc = discount(cdaq_soc, disc_cycle),
  cost_daq_new = discount(cdaq_new, disc_cycle),
  cost_dadmin = discount(cadmin, disc_cycle),
  cost_dman = discount(cman_pf, disc_cycle),  
  cost_ae = risk_ae * uc_ae,
  cost_subs = 0,
  cost_total = cost_daq_soc + cost_daq_new + cost_dadmin + cost_dman + cost_ae + cost_subs,
  # Health utility, QALYs and life years
  pf_year = discount(cycle_years, disc_cycle),
  life_year = discount(cycle_years, disc_cycle),
  qaly = discount(cycle_years * u_pf, disc_cycle)
  )

# Define PD states
state_PD <- heemod::define_state(
  # Costs for the state
  cost_daq_soc = 0,
  cost_daq_new = 0,
  cost_dadmin = 0,
  cost_dman = discount(cman_pd, disc_cycle),  
  cost_ae = 0,
  cost_subs = discount(csubs, disc_cycle),
  cost_total = cost_daq_soc + cost_daq_new + cost_dadmin + cost_dman + cost_ae + cost_subs,
  # Health utility, QALYs and life years
  pf_year = 0,
  life_year = heemod::discount(cycle_years, disc_cycle),
  qaly = heemod::discount(cycle_years * u_pd, disc_cycle)
  )

# Define Death state
state_Death <- heemod::define_state(
  # Costs are zero
  cost_daq_soc = 0,
  cost_daq_new = 0,
  cost_dadmin = 0,
  cost_dman = 0,
  cost_ae = 0,
  cost_subs = 0,
  cost_total = cost_daq_soc + cost_daq_new + cost_dadmin + cost_dman + cost_ae + cost_subs,
  # Health outcomes are zero
  pf_year = 0,
  life_year = 0,
  qaly = 0,
)

# Define strategy for SoC
strat_soc <- heemod::define_strategy(
    transition = psm_soc,
    "PF" = state_PF,
    "PD" = state_PD,
    "Death" = state_Death
  )

# Define strategy for new
strat_new <- heemod::define_strategy(
  transition = psm_new,
  "PF" = state_PF,
  "PD" = state_PD,
  "Death" = state_Death
)

# Create heemod model
heemodel <- heemod::run_model(
  soc = strat_soc,
  new = strat_new,
  parameters = params,
  cycles = Ncycles,
  cost = cost_total,
  effect = qaly,
  init = c(1, 0, 0),
  method = "life-table"
)
```

```{r dynpricing, include=FALSE}
# Dates
# Date of calculation = 1 September 2025
doc <- lubridate::ymd("20250901")
# Date of LOE for SoC = 1 January 2028
loe_soc_start <- lubridate::ymd("20280101")
# Maturation of SoC prices by LOE + 1 year, i.e. = 1 January 2029
loe_soc_end <- lubridate::ymd("20290101") 
# Date of LOE for new treatment = 1 January 2031
loe_new_start <- lubridate::ymd("20310101")
# Maturation of new treatment prices by LOE + 1 year, i.e. = 1 January 2032
loe_new_end <- lubridate::ymd("20320101") 

# Effect of LoEs on prices once mature
loe_effect_soc <- 0.7
loe_effect_new <- 0.5

# Calculation of weeks since DoC for LoEs and price maturities
wk_start_soc <- floor((loe_soc_start-doc) / lubridate::dweeks(1))
wk_end_soc <- floor((loe_soc_end-doc) / lubridate::dweeks(1))
wk_start_new <- floor((loe_new_start-doc) / lubridate::dweeks(1))
wk_end_new <- floor((loe_new_end-doc) / lubridate::dweeks(1))

# Price maturity times
wk_mature_soc <- wk_end_soc - wk_start_soc
wk_mature_new <- wk_end_new - wk_start_new

# Create a tibble of price indices of length 2T, then pull out columns as needed
# We only need of length T for now, but need of length 2T for future calculations later
pricetib <- dplyr::tibble(
  model_time = 1:(2*Ncycles),
  model_year = model_time * cycle_years,
  static = 1,
  geninfl = (1 + infl_cycle)^(model_time - 1),
  loef_soc = pmin(pmax(model_time - wk_start_soc, 0), wk_mature_soc) / wk_mature_soc,
  loef_new = pmin(pmax(model_time - wk_start_new, 0), wk_mature_new) / wk_mature_new,
  dyn_soc = geninfl * (1 - loe_effect_soc * loef_soc),
  dyn_new = geninfl * (1 - loe_effect_new * loef_new)
)

# Price indices required for calculations
prices_oth <- pricetib$geninfl
prices_static <- pricetib$static
prices_dyn_soc <- pricetib$dyn_soc
prices_dyn_new <- pricetib$dyn_new
```

```{r dynuptake, include=FALSE}
# Time for uptake to occur
uptake_years <- 2

# Uptake vector for non-dynamic uptake
uptake_single <- c(1, rep(0, Ncycles-1))

# Uptake vector for dynamic uptake
uptake_weeks <- round(uptake_years / cycle_years)
share_multi <- c((1:uptake_weeks)/uptake_weeks, rep(1, Ncycles-uptake_weeks))
uptake_multi <- rep(1, Ncycles) * share_multi
```

```{r static2, include=FALSE}
# Pull out the payoffs of interest from oncpsm
payoffs <- get_dynfields(
    heemodel = heemodel,
    payoffs = c("cost_daq_new", "cost_daq_soc", "cost_total", "qaly", "life_year"),
    discount = "disc"
    ) |>
    dplyr::mutate(
      model_years = model_time * cycle_years,
      # Derive costs other than drug acquisition, as at time zero
      cost_nondaq = cost_total - cost_daq_new - cost_daq_soc,
      # ... and at the start of each timestep
      cost_nondaq_rup = cost_total_rup - cost_daq_new_rup - cost_daq_soc_rup
    )

# Create and view dataset for SoC
hemout_soc <- payoffs |> dplyr::filter(int=="soc")
head(hemout_soc)

# Create and view dataset for new intervention
hemout_new <- payoffs |> dplyr::filter(int=="new")
head(hemout_new)
```

The underlying health economic model is built as described in `vignette("cost-effectiveness-applications")`. We require additional coding for the budget impact evaluation.

```{r bimshare}
# BIM settings
bi_horizon_yrs <- 5
bi_horizon_wks <- round(bi_horizon_yrs / cycle_years)
bi_discount <- 0

# Newly eligible patients
newly_eligible <- rep(1, Ncycles)

# Time for uptake to occur
uptake_years <- 2
uptake_weeks <- round(uptake_years / cycle_years)

# Market share of new intervention
share_multi <- c((1:uptake_weeks)/uptake_weeks, rep(1, Ncycles-uptake_weeks))

# Newly eligible patients receiving each intervention, "world with"
uptake_new <- newly_eligible * share_multi
uptake_soc <- newly_eligible - uptake_new
```

## Results

### Scenario 1: No dynamic uptake or pricing

This scenario assumes static uptake, i.e. that uptake is immediate and 100\% of eligible patients. This is an unconventional approach for a budget impact analysis. The analysis also assumes static prices, i.e. we assume the prices of existing resources remain unchanged from now in the horizon of the budget impact model. First we calculate costs with the SoC, in the 'world without' the new intervention.

```{r bim_wout1}
# World without new intervention

# SoC, drug acquisition costs
wout1_soc_daqcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_soc$cost_daq_soc_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# SoC, other costs
wout1_soc_othcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_soc$cost_nondaq_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# Total budgetary costs
budget_wout1_soc <- wout1_soc_daqcost + wout1_soc_othcost
budget_wout1_new <- 0
budget_wout1 <- budget_wout1_soc
```

The total budgetary costs in the world without are $`r total(budget_wout1)` in respect of `r uptake(budget_wout1)` patients.

Next we calculate costs with the new treatment, in the 'world with' the new intervention.

```{r bim_with1}
# World with: SoC are zero

# New intervention, drug acquisition costs
with1_new_daqcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_new$cost_daq_new_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# New intervention, other costs
with1_new_othcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_new$cost_nondaq_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# Total
budget_with1_soc <- 0
budget_with1_new <- with1_new_daqcost + with1_new_othcost
budget_with1 <- budget_with1_new
```

The budgetary costs in the world with the new intervention are $`r total(budget_with1)`$. Finally, we derive the budget impact as the difference in costs.

```{r bim_iwith1}
# Budget impact
bi1 <- budget_with1 - budget_wout1
summary(bi1)
```

The total budget impact is $`r total(bi1)`, representing an increase of `r total(bi1)/total(budget_wout1) *100`%.

### Scenario 2: Dynamic pricing, no dynamic uptake

Scenario 2 adjusts scenario 1 by assuming instead dynamic pricing of the new intervention and SoC. Uptake remains modeled in the unconventional static manner. As before, we first calculate the costs of the SoC in the 'world without' the new intervention.

```{r bim_wout2}
# World without new intervention

# SoC, drug acquisition costs
wout2_soc_daqcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_soc$cost_daq_soc_rup,
    horizon = bi_horizon_wks,
    prices = prices_dyn_soc,
    discrate = bi_discount
    )

# SoC, other costs
wout2_soc_othcost <- wout1_soc_othcost

# Total budgetary costs
budget_wout2_soc <- wout2_soc_daqcost + wout2_soc_othcost
budget_wout2_new <- 0
budget_wout2 <- budget_wout2_soc
```

The total budgetary costs in the world without are $`r total(budget_wout2)` in respect of `r uptake(budget_wout2)` patients.

Next we derive the costs of the new intervention, in the 'world with' the new intervention.

```{r bim_with2}
# World with: SoC are zero

# New intervention, drug acquisition costs
with2_new_daqcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_new$cost_daq_new_rup,
    horizon = bi_horizon_wks,
    prices = prices_dyn_new,
    discrate = bi_discount
    )

# New intervention, other costs
with2_new_othcost <- with1_new_othcost

# Total
budget_with2_soc <- 0
budget_with2_new <- with2_new_daqcost + with2_new_othcost
budget_with2 <- budget_with2_new
```

The budgetary costs in the world with the new intervention are $`r total(budget_with2)`$. Finally again, we derive budget impact as the difference.

```{r bim_iwith2}
# Budget impact
bi2 <- budget_with2 - budget_wout2
summary(bi2)
```
The total budget impact is $`r total(bi2)`, representing an increase of `r total(bi2)/total(budget_wout2) *100`%.

### Scenario 3: Dynamic uptake, not dynamic pricing

This scenario includes dynamic uptake, which is conventional in budget impact analysis, but assumes static prices, i.e. we assume the prices of existing resources remain unchanged from now in the horizon of the budget impact model. Let us use that function to calculate budgetary costs for the world without the new intervention.

```{r bim_wout3}
# World without new intervention
# SoC, drug acquisition costs
wout3_soc_daqcost <- wout1_soc_daqcost
# SoC, other costs
wout3_soc_othcost <- wout1_soc_othcost
# Total budgetary costs
budget_wout3 <- budget_wout3_soc <- wout3_soc_daqcost + wout3_soc_othcost
```

The total budgetary costs in the world without are $`r total(budget_wout3)` in respect of `r uptake(budget_wout3)` patients.

Let us now calculate the budgetary costs in the world with the new intervention.

```{r bim_with3}
# World with

# SoC, drug acquisition costs
with3_soc_daqcost <- dynpv(
    uptakes = uptake_soc,
    payoffs = hemout_soc$cost_daq_soc_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# SoC, other costs
with3_soc_othcost <- dynpv(
    uptakes = uptake_soc,
    payoffs = hemout_soc$cost_nondaq_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# New intervention, drug acquisition costs
with3_new_daqcost <- dynpv(
    uptakes = uptake_new,
    payoffs = hemout_new$cost_daq_new_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# New intervention, other costs
with3_new_othcost <- dynpv(
    uptakes = uptake_new,
    payoffs = hemout_new$cost_nondaq_rup,
    horizon = bi_horizon_wks,
    prices = prices_static,
    discrate = bi_discount
    )

# Total
budget_with3_soc <- with3_soc_daqcost + with3_soc_othcost
budget_with3_new <- with3_new_daqcost + with3_new_othcost
budget_with3 <- budget_with3_soc + budget_with3_new
```

Note the warning provided by `dynamicpv`. This is because the uptake vector for SoC, when trimmed of zeroes after uptake stops, has a different (shorter) length than the uptake vector for the new intervention. The calculation is still correct. However, the function is flagging for the user the different uptake vectors being used for different present value calculations.

```{r warnlength}
# The uptake vector for the new intervention is long
length(trim_vec(uptake_new))

# The uptake vector for the SoC is short, once trimmed of excess zeros
length(trim_vec(uptake_soc))
```

The budgetary costs in the world with the new intervention are $`r total(budget_with3)`, comprising $`r total(budget_with3_soc)` in respect of the costs of `r uptake(budget_with3_soc)` patients being treated with the SoC, and $`r total(budget_with3_new)` in respect of the costs of `r uptake(budget_with3_new)` patients being treated with the SoC. 

```{r bim_iwith3}
# Budget impact
bi3_soc <- budget_with3_soc - budget_wout3_soc
bi3_new <- budget_with3_new
bi3 <- budget_with3 - budget_wout3

summary(bi3)
```

The total budget impact is $`r total(bi3)`, representing an increase of `r total(bi3)/total(budget_wout3) *100`%.

### Scenario 4: Dynamic pricing and uptake

Now let us recalculate the budget impact, assuming dynamic pricing in drug acquisition costs. This is simple with `dynamicpv()::dynpv()` because we just change the `prices` argument from `prices_static` to either `prices_dyn_soc` or `prices_dyn_new` for the drug acquisition costs. We will keep other costs unchanged.

```{r bim_wout4}
# World without new intervention

# SoC, drug acquisition costs
wout4_soc_daqcost <- dynpv(
    uptakes = newly_eligible,
    payoffs = hemout_soc$cost_daq_soc_rup,
    horizon = bi_horizon_wks,
    prices = prices_dyn_soc,
    discrate = bi_discount
    )

# SoC, other costs - unchanged from static calculations
wout4_soc_othcost <- wout3_soc_othcost

# Total budgetary costs
budget_wout4 <- budget_wout4_soc <- wout4_soc_daqcost + wout4_soc_othcost
```

The total budgetary costs in the world without are $`r total(budget_wout4)` in respect of `r uptake(budget_wout4)` patients.

Let us now calculate the budgetary costs in the world with the new intervention.

```{r bim_with4}
# World with

# SoC, drug acquisition costs
with4_soc_daqcost <- dynpv(
    uptakes = uptake_soc,
    payoffs = hemout_soc$cost_daq_soc_rup,
    horizon = bi_horizon_wks,
    prices = prices_dyn_soc,
    discrate = bi_discount
    )

# SoC, other costs
with4_soc_othcost <- with3_soc_othcost

# New intervention, drug acquisition costs
with4_new_daqcost <- dynpv(
    uptakes = uptake_new,
    payoffs = hemout_new$cost_daq_new_rup,
    horizon = bi_horizon_wks,
    prices = prices_dyn_new,
    discrate = bi_discount
    )

# New intervention, other costs
with4_new_othcost <- with3_new_othcost

# Total
budget_with4_soc <- with4_soc_daqcost + with4_soc_othcost
budget_with4_new <- with4_new_daqcost + with4_new_othcost
budget_with4 <- budget_with4_soc + budget_with4_new
```

Notice that there is a similar warning as earlier. The budgetary costs in the world with the new intervention are $`r total(budget_with4)`, comprising $`r total(budget_with4_soc)` in respect of the costs of `r uptake(budget_with4_soc)` patients being treated with the SoC, and $`r total(budget_with2_new)` in respect of the costs of `r uptake(budget_with4_new)` patients being treated with the new treatment. 

```{r bim_iwith4}
# Budget impact
bi4_soc <- budget_with4_soc - budget_wout4_soc
bi4_new <- budget_with4_new
bi4 <- budget_with4 - budget_wout4

summary(bi4)
```

The total budget impact is $`r total(bi4)`, representing an increase of `r total(bi4)/total(budget_wout4) *100`%.

### Summary

|    |  | Scenario 1 | Scenario 2 | Scenario 3 | Scenario 4 |
|:---|:--|------------|------------|------------|------------|
| Dynamic pricing? || No | Yes | No | Yes |
| Dynamic uptake? || No | No | Yes | Yes |
| World without new intervention || ||  |  |
| | Standard of Care |`r total(budget_wout1_soc)`  | `r total(budget_wout2_soc)` | `r total(budget_wout3_soc)` | `r total(budget_wout4_soc)`  |
| | New intervention | 0 | 0 | 0 | 0 |
| | Total | `r total(budget_wout1)` | `r total(budget_wout2)` | `r total(budget_wout3)` | `r total(budget_wout4)` |
| World with new intervention ||||  |  |
| | Standard of Care | 0 | 0 | `r total(budget_with3_soc)`  | `r total(budget_with4_soc)`  |
| | New intervention | `r total(budget_with1_new)` | `r total(budget_with2_new)` | `r total(budget_with3_new)` | `r total(budget_with4_new)` |
| | Total | `r total(budget_with1)` | `r total(budget_with2)` | `r total(budget_with3)` | `r total(budget_with4_new)`  |
| Budget impact || ||  |  |
| | Standard of Care | `r -total(budget_wout1_soc)` | `r -total(budget_wout2_soc)` | `r total(bi3_soc)` | `r total(bi4_soc)`  |
| | New intervention | `r total(budget_with1_new)` | `r total(budget_with2_new)` | `r total(bi3_new)` | `r total(bi4_new)`  |
| | Absolute impact | `r total(bi1)`| `r total(bi2)` | `r total(bi3)` | `r total(bi4)`  |
| | Relative impact (%) | `r ((total(bi1))/(total(budget_wout1))*100)`% | `r ((total(bi2))/(total(budget_wout2))*100)`% | `r ((total(bi3))/(total(budget_wout3))*100)`% | `r ((total(bi4))/(total(budget_wout4))*100)`%  |

: Budget Impact model result by scenario

## Discussion

- In both cases, the budget impact is large, with budgetary costs more than doubling with the introduction of the new intervention, over the time horizon of interest (`r bi_horizon_yrs` years).
- Dynamic pricing leads to greater anticipated costs of the new intervention and lower expected budgetary costs of the SoC, over the time horizon of interest. Accordingly, in this example, the budget impact is greater in the dynamic pricing scenario.
- Further stratification of the results, by intervention received, time period, cost component etc may reveal further insights. These are possible from the results calculated and presented by `dynamicpv::dynpv()` but not shown in this simple illustration.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/future.R
\name{futurepv}
\alias{futurepv}
\title{Calculate present value for a payoff in a single cohort with dynamic pricing across multiple timepoints}
\usage{
futurepv(tzero = 0, payoffs, prices, discrate)
}
\arguments{
\item{tzero}{Time at the date of calculation, to be used in lookup in prices vector}

\item{payoffs}{Vector of payoffs of interest (numeric vector)}

\item{prices}{Vector of price indices through the time horizon of interest}

\item{discrate}{Discount rate per timestep, corresponding to price index}
}
\value{
A tibble of class "dynpv" with the following columns:
\itemize{
\item \code{j}: Time at which patients began treatment
\item \code{k}: Time since patients began treatment
\item \code{l}: Time offset for the price index (from \code{tzero})
\item \code{t}: Equals \eqn{j+k-1}
\item \code{uj}: Uptake of patients beginning treatment at time \eqn{j} (from \code{uptakes})
\item \code{pk}: Cashflow amount in today's money in respect of patients at time \eqn{k} since starting treatment (from \code{payoffs})
\item \code{R}: Index of prices over time \eqn{l+t} (from \code{prices})
\item \code{v}: Discounting factors, \eqn{(1+i)^{1-t}}, where \code{i} is the discount rate per timestep
\item \code{pv}: Present value, \eqn{PV(j,k,l)}
}
}
\description{
Present value of a series of payoffs for a single given cohort, entering at given future time, allowing for dynamic pricing. This function is a wrapper for \code{\link[=dynpv]{dynpv()}} restricted to evaluation of a single cohort.
}
\examples{
library(dplyr)

# Obtain dataset
democe <- get_dynfields(
   heemodel = oncpsm,
   payoffs = c("cost_daq_new", "cost_total", "qaly"),
   discount = "disc"
   )

# Obtain discount rate
discrate <- get_param_value(oncpsm, "disc")

# Obtain payoff vector of interest
payoffs <- democe |>
   filter(int=="new") |>
   mutate(cost_oth_rup = cost_total_rup - cost_daq_new_rup)
Nt <- nrow(payoffs)

# Run calculation for times 0-9
fpv <- futurepv(
  tzero = (0:9)*52,
  payoffs = payoffs$cost_oth_rup,
  prices = 1.001^(1:(2*Nt)-1), # Approx 5.3\% every 52 steps
  discrate = 0.001 + discrate
)
fpv
summary(fpv)
}
\seealso{
\code{\link[=dynpv]{dynpv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dynamic.R
\name{dynpv}
\alias{dynpv}
\title{Present values with dynamic pricing and dynamic uptake}
\usage{
dynpv(
  uptakes = 1,
  payoffs,
  horizon = length(payoffs),
  tzero = 0,
  prices = rep(1, length(payoffs) + tzero),
  discrate = 0
)
}
\arguments{
\item{uptakes}{Vector of patient uptake over time}

\item{payoffs}{Vector of payoffs of interest (numeric vector)}

\item{horizon}{Time horizon for the calculation (length must be less than or equal to the length of payoffs)}

\item{tzero}{Time at the date of calculation, to be used in lookup in prices vector}

\item{prices}{Vector of price indices through the time horizon of interest}

\item{discrate}{Discount rate per timestep, corresponding to price index}
}
\value{
A tibble of class "dynpv" with the following columns:
\itemize{
\item \code{j}: Time at which patients began treatment
\item \code{k}: Time since patients began treatment
\item \code{l}: Time offset for the price index (from \code{tzero})
\item \code{t}: Equals \eqn{j+k-1}
\item \code{uj}: Uptake of patients beginning treatment at time \eqn{j} (from \code{uptakes})
\item \code{pk}: Cashflow amount in today's money in respect of patients at time \eqn{k} since starting treatment (from \code{payoffs})
\item \code{R}: Index of prices over time \eqn{l+t} (from \code{prices})
\item \code{v}: Discounting factors, \eqn{(1+i)^{1-t}}, where \code{i} is the discount rate per timestep
\item \code{pv}: Present value, \eqn{PV(j,k,l)}
}
}
\description{
Calculate present value for a payoff with dynamic (lifecycle) pricing and dynamic uptake (stacked cohorts).
}
\details{
Suppose payoffs in relation to patients receiving treatment (such as costs or health outcomes) occur over timesteps \eqn{t=1, ..., T}. Let us partition time as follows.
\itemize{
\item Suppose \eqn{j=1,...,T} indexes the time at which the patient begins treatment.
\item Suppose \eqn{k=1,...,T} indexes time since initiating treatment.
}

In general, \eqn{t=j+k-1}, and we are interested in the set of \eqn{(j,k)} such that \eqn{1 \leq t \leq T}.

For example, \eqn{t=3} comprises:
\itemize{
\item patients who are in the third timestep of treatment that began in timestep 1: (j,k)=(1,3);
\item patients who are in the second timestep of treatment that began in timestep 2, (j,k)=(2,2); and
\item patients who are in the first timestep of treatment that began in timestep 3, (j,k)=(3,1)
}

The \href{https://en.wikipedia.org/wiki/Present_value}{Present Value} of a cashflow \eqn{p_k} for the \eqn{u_j} patients who began treatment at time \eqn{j} and who are in their \eqn{k}th timestep of treatment is as follows
\deqn{PV(j,k,l) = u_j \cdot p_k \cdot R_{j+k+l-1} \cdot (1+i)^{2-j-k}}
where \eqn{i} is the risk-free discount rate per timestep, \eqn{p_k} is the cashflow amount in today’s money, and \eqn{p_k \cdot R_{j+k+l-1}} is the nominal amount of the cashflow at the time it is incurred, allowing for an offset of \eqn{l = tzero}.

The total present value, \eqn{TPV(l)}, is therefore the sum over all \eqn{j} and \eqn{k} within the time horizon \eqn{T}, namely:
\deqn{TPV(l) = \sum_{j=1}^{T} \sum_{k=1}^{T-j+1} PV(j,k, l) \\
\;
= \sum_{j=1}^{T} \sum_{k=1}^{T-j+1} u_j \cdot p_k \cdot R_{l+j+k-1} \cdot (1+i)^{2-j-k}}

This function calculates \eqn{PV(j,k,l)} for all values of \eqn{j}, \eqn{k} and \eqn{l}, and returns this in a tibble.
}
\examples{
# Simple example
pv1 <- dynpv(
   uptakes = (1:10), # 1 patient uptakes in timestep 1, 2 patients in timestep 2, etc
   tzero = c(0,5), # Calculations are performed with prices at times 0 and 5
   payoffs = 90 + 10*(1:10), # Payoff vector of length 10 = (100, 110, ..., 190)
   prices = 1.02^((1:15)-1), # Prices increase at 2\\% per timestep in future
   discrate = 0.05 # The nominal discount rate is 5\\% per timestep;
                   # the real discount rate per timestep is 3\\% (=5\\% - 3\\%)
)
pv1
summary(pv1)

# More complex example, using cashflow output from a heemod model

# Obtain dataset
democe <- get_dynfields(
   heemodel = oncpsm,
   payoffs = c("cost_daq_new", "cost_total", "qaly"),
   discount = "disc"
   )

# Obtain short payoff vector of interest
payoffs <- democe |>
   dplyr::filter(int=="new", model_time<11) |>
   dplyr::mutate(cost_oth = cost_total - cost_daq_new)

# Example calculation
pv2 <- dynpv(
   uptakes = rep(1, nrow(payoffs)),
   payoffs = payoffs$cost_oth,
   prices = 1.05^(0:(nrow(payoffs)-1)),
   discrate = 0.08
)
summary(pv2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oop.R
\name{uptake}
\alias{uptake}
\title{Total number of uptaking patients}
\usage{
uptake(df)
}
\arguments{
\item{df}{Tibble of class "dynpv" created by \code{\link[=dynpv]{dynpv()}} or \code{\link[=futurepv]{futurepv()}}}
}
\value{
A number or tibble
}
\description{
Total number of uptaking patients, calculated as the sum of the \code{uptake} input to \code{\link[=dynpv]{dynpv()}}, or \eqn{\sum_{j=1}^T u_j}
}
\seealso{
\code{\link[=dynpv]{dynpv()}}, \code{\link[=futurepv]{futurepv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oop.R
\name{summary.dynpv}
\alias{summary.dynpv}
\title{Summarize a dynpv object}
\usage{
\method{summary}{dynpv}(object, ...)
}
\arguments{
\item{object}{Tibble of class "dynpv" created by \code{\link[=dynpv]{dynpv()}} or \code{\link[=futurepv]{futurepv()}}}

\item{...}{Currently unused}
}
\value{
A list of class "dynpv_summary" with the following elements:
\itemize{
\item \code{ncoh}: Number of cohorts of uptaking patients, from \code{\link[=ncoh]{ncoh()}}
\item \code{ntimes}: Number of times at which present value calculations are performed, from \code{\link[=ntimes]{ntimes()}}
\item \code{uptake}: Total number of uptaking patients, from \code{\link[=uptake]{uptake()}}
\item \code{sum_by_coh}: Present value for each uptake cohort and calculation time, from \code{\link[=sum_by_coh]{sum_by_coh()}}
\item \code{total}: Total present value, from \code{\link[=total]{total()}}
\item \code{mean}: Mean present value per uptaking patient, from \code{\link[=mean]{mean()}}, equal to \code{total}/\code{uptake}.
}
}
\description{
Summarize a dynpv object
}
\seealso{
\code{\link[=dynpv]{dynpv()}}, \code{\link[=futurepv]{futurepv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oop.R
\name{total}
\alias{total}
\title{Total present value}
\usage{
total(df)
}
\arguments{
\item{df}{Tibble of class "dynpv" created by \code{\link[=dynpv]{dynpv()}} or \code{\link[=futurepv]{futurepv()}}}
}
\value{
A number or tibble
}
\description{
Sum of the Present Value, by time at which the calculation is performed (\code{tzero} input to \code{\link[=dynpv]{dynpv()}})
}
\details{
The \href{https://en.wikipedia.org/wiki/Present_value}{Present Value} of a cashflow \eqn{p_k} for the \eqn{u_j} patients who began treatment at time \eqn{j} and who are in their \eqn{k}th timestep of treatment is as follows
\deqn{PV(j,k,l) = u_j \cdot p_k \cdot R_{j+k+l-1} \cdot (1+i)^{2-j-k}}
where \eqn{i} is the risk-free discount rate per timestep, \eqn{p_k} is the cashflow amount in today’s money, and \eqn{p_k \cdot R_{j+k+l-1}} is the nominal amount of the cashflow at the time it is incurred, allowing for an offset of \eqn{l = tzero}.

The total present value by time at which the calculation is performed, \eqn{TPV(l)}, is therefore the sum of \eqn{PV(j,k,l)} over all \eqn{j} and \eqn{k} within the time horizon \eqn{T}, namely:
\deqn{TPV(l) = \sum_{j=1}^{T} \sum_{k=1}^{T-j+1} PV(j,k, l) \\
\;
= \sum_{j=1}^{T} \sum_{k=1}^{T-j+1} u_j \cdot p_k \cdot R_{l+j+k-1} \cdot (1+i)^{2-j-k}}
}
\seealso{
\code{\link[=dynpv]{dynpv()}}, \code{\link[=futurepv]{futurepv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oop.R
\name{ncoh}
\alias{ncoh}
\title{Number of cohorts of uptaking patients}
\usage{
ncoh(df)
}
\arguments{
\item{df}{Tibble of class "dynpv" created by \code{\link[=dynpv]{dynpv()}} or \code{\link[=futurepv]{futurepv()}}}
}
\value{
A number
}
\description{
Number of cohorts of uptaking patients, calculated as the length of the \code{uptakes} input to \code{\link[=dynpv]{dynpv()}}
}
\seealso{
\code{\link[=dynpv]{dynpv()}}, \code{\link[=futurepv]{futurepv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{get_param_value}
\alias{get_param_value}
\title{Obtain parameter value(s) from a heemod output}
\usage{
get_param_value(heemodel, param)
}
\arguments{
\item{heemodel}{A health economic model object from the \emph{heemod} package (see \link[heemod:heemod-package]{heemod::heemod-package}).}

\item{param}{Name of parameter to extract from the heemod model}
}
\value{
Value of the parameter from the heemod model
}
\description{
Obtain parameter value(s) from a heemod output
}
\examples{
get_param_value(
   heemodel = oncpsm,
   param = "disc"
   )
}
\seealso{
\link[heemod:heemod-package]{heemod::heemod-package}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dynamic.R
\name{trim_vec}
\alias{trim_vec}
\title{Trim the tailing zeroes from a long vector}
\usage{
trim_vec(vec)
}
\arguments{
\item{vec}{Vector. Final elements may be zero.}
}
\value{
A vector whose length is shorter than the original, if there were trailing zero elements
}
\description{
Trim the tailing zeroes from a long vector
}
\examples{
trim_vec(c(1:10, rep(0,3)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oop.R
\name{sum_by_coh}
\alias{sum_by_coh}
\title{Present value for each uptake cohort and calculation time}
\usage{
sum_by_coh(df)
}
\arguments{
\item{df}{Tibble of class "dynpv" created by \code{\link[=dynpv]{dynpv()}} or \code{\link[=futurepv]{futurepv()}}}
}
\value{
A number or tibble
}
\description{
Calculates the sum of the Present Value by uptake cohort (\code{j}) and time at which the calculation is performed (\code{tzero} input to \code{\link[=dynpv]{dynpv()}})
}
\details{
The \href{https://en.wikipedia.org/wiki/Present_value}{Present Value} of a cashflow \eqn{p_k} for the \eqn{u_j} patients who began treatment at time \eqn{j} and who are in their \eqn{k}th timestep of treatment is as follows
\deqn{PV(j,k,l) = u_j \cdot p_k \cdot R_{j+k+l-1} \cdot (1+i)^{2-j-k}}
where \eqn{i} is the risk-free discount rate per timestep, \eqn{p_k} is the cashflow amount in today’s money, and \eqn{p_k \cdot R_{j+k+l-1}} is the nominal amount of the cashflow at the time it is incurred, allowing for an offset of \eqn{l = tzero}.

This method returns \eqn{\sum_{k=1}^{T-j+1} PV(j,k,l)} for each value of \eqn{j} and \eqn{l}, where \eqn{T} is the time horizon of the calculation.
}
\seealso{
\code{\link[=dynpv]{dynpv()}}, \code{\link[=futurepv]{futurepv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{oncpsm}
\alias{oncpsm}
\title{Heemod cost-effectiveness model example}
\format{
\subsection{\code{oncpsm}}{

A heemod object
}
}
\source{
Created based on assumptions.
}
\usage{
oncpsm
}
\description{
An example three state cost-effectiveness model in oncology built using \code{\link[heemod:heemod-package]{heemod::heemod-package()}} according to the assumptions and specification in the accompanying paper.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{get_dynfields}
\alias{get_dynfields}
\title{Helper function to get a tibble of the relevant fields from heemod output}
\usage{
get_dynfields(heemodel, payoffs, discount, fname = NA)
}
\arguments{
\item{heemodel}{A health economic model object from the \emph{heemod} package (see \link[heemod:heemod-package]{heemod::heemod-package}).}

\item{payoffs}{Field names of payoffs of interest (string vector)}

\item{discount}{Name of parameter providing discount rate per cycle (string)}

\item{fname}{Export data to a .CSV file of this name, if given (character)}
}
\value{
Tibble of payoffs taken from the heemod model, by intervention and model timestep (\code{model_time}).

The field \code{vt} is calculated as \code{(1+i)^(1-model_time)}, where \code{i} is the discount rate per model timestep set in the \emph{heemod} model through the parameter \code{disc_cycle}. This can be useful in 'rolling-up' payoff values to the timestep in which they were incurred.

An additional set of payoffs (identified with a "_rup" suffix) provides calculations of the payoffs as at the start of the timestep in which they were incurred, i.e. original payoff / \code{vt}.
}
\description{
Helper function to get a tibble of the relevant fields from heemod output
}
\examples{
democe <- get_dynfields(
   heemodel = oncpsm,
   payoffs = c("cost_daq_new", "cost_total", "qaly"),
   discount = "disc"
   )
head(democe)
}
\seealso{
\link[heemod:heemod-package]{heemod::heemod-package}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oop.R
\name{ntimes}
\alias{ntimes}
\title{Number of times at which present value calculations are performed}
\usage{
ntimes(df)
}
\arguments{
\item{df}{Tibble of class "dynpv" created by \code{\link[=dynpv]{dynpv()}} or \code{\link[=futurepv]{futurepv()}}}
}
\value{
A number
}
\description{
Number of times at which present value calculations are performed, calculated as the length of the \code{tzero} input to \code{\link[=dynpv]{dynpv()}}
}
\seealso{
\code{\link[=dynpv]{dynpv()}}, \code{\link[=futurepv]{futurepv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oop.R
\name{addprod}
\alias{addprod}
\title{Method to add two dynpv objects together}
\usage{
addprod(e1, e2, mult)
}
\arguments{
\item{e1}{First "dynpv" object}

\item{e2}{Second "dynpv" object}

\item{mult}{Numeric

Present value is present value from \code{e1} plus \code{mult} times the present value
from \code{e2}. Total uptake is the uptake from \code{e1} plus \code{mult} times the uptake
from \code{e2}. Take care of this when using \verb{$mean} of the summed object.}
}
\value{
S3 object of class "dynpv"
}
\description{
Add together two objects each of S3 class "dynpv": e1 + mult * e2
}
\seealso{
\code{\link[=dynpv]{dynpv()}}, \code{\link[=futurepv]{futurepv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dynamicpv-package.R
\docType{package}
\name{dynamicpv-package}
\alias{dynamicpv}
\alias{dynamicpv-package}
\title{dynamicpv: Evaluates Present Values and Health Economic Models with Dynamic Pricing and Uptake}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

The goal of 'dynamicpv' is to provide a simple way to calculate (net) present values and outputs from health economic models (especially cost-effectiveness and budget impact) in discrete time that reflect dynamic pricing and dynamic uptake. Dynamic pricing is also known as life cycle pricing; dynamic uptake is also known as multiple or stacked cohorts, or dynamic disease prevalence. Shafrin (2024) \doi{10.1515/fhep-2024-0014} provides an explanation of dynamic value elements, in the context of Generalized Cost Effectiveness Analysis, and Puls (2024) \doi{10.1016/j.jval.2024.03.006} reviews challenges of incorporating such dynamic value elements. This package aims to reduce those challenges.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://MSDLLCpapers.github.io/dynamicpv/}
  \item \url{https://github.com/MSDLLCpapers/dynamicpv}
  \item Report bugs at \url{https://github.com/MSDLLCpapers/dynamicpv/issues}
}

}
\author{
\strong{Maintainer}: Dominic Muston \email{dom.muston@gmail.com} (\href{https://orcid.org/0000-0003-4876-7940}{ORCID})

Other contributors:
\itemize{
  \item John Blischak (\href{https://orcid.org/0000-0003-2634-9879}{ORCID}) [contributor]
  \item Merck & Co., Inc., Rahway, NJ, USA and its affiliates [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oop.R
\name{mean.dynpv}
\alias{mean.dynpv}
\title{Mean present value per uptaking patient}
\usage{
\method{mean}{dynpv}(x, ...)
}
\arguments{
\item{x}{Tibble of class "dynpv" created by \code{\link[=dynpv]{dynpv()}} or \code{\link[=futurepv]{futurepv()}}}

\item{...}{Currently unused}
}
\value{
A number or tibble
}
\description{
Mean of the Present Value per uptaking patient, by time at which the calculation is performed (\code{tzero} input to \code{\link[=dynpv]{dynpv()}}).
}
\details{
This is equal to \code{\link[=total]{total()}} divided by \code{\link[=uptake]{uptake()}}.
}
\seealso{
\code{\link[=dynpv]{dynpv()}}, \code{\link[=futurepv]{futurepv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oop.R
\name{-.dynpv}
\alias{-.dynpv}
\title{Method to subtract one dynpv object from another}
\usage{
\method{-}{dynpv}(e1, e2)
}
\arguments{
\item{e1}{First "dynpv" object}

\item{e2}{Second "dynpv" object}
}
\value{
S3 object of class "dynpv"
}
\description{
Subtract one object of S7 class "dynpv" from another
}
\details{
Present value of \code{e1-e2} is the present values from \code{e1} less that from \code{e2}.
Total uptake of \code{e1-e2} is the uptake from \code{e1} less that from \code{e2}. Take
care of this when using \verb{$mean} of the summed object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oop.R
\name{+.dynpv}
\alias{+.dynpv}
\title{Method to add two dynpv objects together}
\usage{
\method{+}{dynpv}(e1, e2)
}
\arguments{
\item{e1}{First "dynpv" object}

\item{e2}{Second "dynpv" object}
}
\value{
S3 object of class "dynpv"
}
\description{
Add together two objects each of class "dynpv"
}
